@@ -1,9 +1,9 @@
 /* Distributed under the OSI-approved BSD 3-Clause License.  See accompanying
-   file Copyright.txt or https://cmake.org/licensing for details.  */
+   file LICENSE.rst or https://cmake.org/licensing for details.  */
 #include "cmCPackGenerator.h"
 
 #include <algorithm>
-#include <cstring>
+#include <iterator>
 #include <memory>
 #include <utility>
 
@@ -51,11 +51,10 @@ cmCPackGenerator::~cmCPackGenerator()
   this->MakefileMap = nullptr;
 }
 
-void cmCPackGenerator::DisplayVerboseOutput(const std::string& msg,
-                                            float progress)
+void cmCPackGenerator::DisplayVerboseOutput(std::string const& msg,
+                                            float /*unused*/)
 {
-  (void)progress;
-  cmCPackLogger(cmCPackLog::LOG_VERBOSE, "" << msg << std::endl);
+  cmCPackLogger(cmCPackLog::LOG_VERBOSE, msg << std::endl);
 }
 
 int cmCPackGenerator::PrepareNames()
@@ -102,18 +101,18 @@ int cmCPackGenerator::PrepareNames()
   std::string pkgFileName =
     cmStrCat(pkgBaseFileName, this->GetOutputExtension());
   // Determine path to the package.
-  std::string pkgFilePath = cmStrCat(pkgDirectory, "/", pkgFileName);
+  std::string pkgFilePath = cmStrCat(pkgDirectory, '/', pkgFileName);
   // Determine top-level directory for packaging.
   std::string topDirectory = cmStrCat(pkgDirectory, "/_CPack_Packages/");
   {
     cmValue toplevelTag = this->GetOption("CPACK_TOPLEVEL_TAG");
     if (toplevelTag) {
-      topDirectory += cmStrCat(toplevelTag, "/");
+      topDirectory += cmStrCat(toplevelTag, '/');
     }
   }
   topDirectory += *this->GetOption("CPACK_GENERATOR");
   // Determine temporary packaging-directory.
-  std::string tmpDirectory = cmStrCat(topDirectory, "/", pkgBaseFileName);
+  std::string tmpDirectory = cmStrCat(topDirectory, '/', pkgBaseFileName);
   // Determine path to temporary package file.
   std::string tmpPkgFilePath = topDirectory + "/" + pkgFileName;
 
@@ -176,14 +175,42 @@ int cmCPackGenerator::PrepareNames()
     return 0;
   }
 
-  // Check algorithm for calculating the checksum of the package.
-  cmValue algoSignature = this->GetOption("CPACK_PACKAGE_CHECKSUM");
-  if (algoSignature) {
-    if (!cmCryptoHash::New(*algoSignature)) {
-      cmCPackLogger(cmCPackLog::LOG_ERROR,
-                    "Cannot recognize algorithm: " << algoSignature
-                                                   << std::endl);
-      return 0;
+  // Check algorithms for calculating the checksum of the package.
+  cmValue algoSignatures = this->GetOption("CPACK_PACKAGE_CHECKSUM");
+  if (cmNonempty(algoSignatures)) {
+    cmList algoList{ algoSignatures };
+    // Workout unique algorithms and duplicates for diagnostic purposes
+    algoList.sort();
+    // Store a copy since std::unique modifies the sequence
+    cmList const sortedAlgoList = algoList;
+    auto const newEnd = std::unique(algoList.begin(), algoList.end());
+
+    if (newEnd != algoList.end()) {
+      cmList duplicatesAlgoList;
+
+      std::set_difference(sortedAlgoList.begin(), sortedAlgoList.end(),
+                          algoList.begin(), newEnd,
+                          std::back_inserter(duplicatesAlgoList));
+      // Make sure to output duplicates a single time even if these appear more
+      // than two times. Exploit the already sorted sequence to determine the
+      // unique elements.
+      duplicatesAlgoList.erase(
+        std::unique(duplicatesAlgoList.begin(), duplicatesAlgoList.end()),
+        duplicatesAlgoList.end());
+
+      cmCPackLogger(cmCPackLog::LOG_WARNING,
+                    "Algorithm specified multiple times: "
+                      << duplicatesAlgoList.join(", ") << std::endl);
+    }
+
+    algoList.erase(newEnd, algoList.end());
+
+    for (std::string const& algo : algoList) {
+      if (!cmCryptoHash::New(algo)) {
+        cmCPackLogger(cmCPackLog::LOG_ERROR,
+                      "Cannot recognize algorithm: " << algo << std::endl);
+        return 0;
+      }
     }
   }
 
@@ -198,8 +225,8 @@ int cmCPackGenerator::InstallProject()
   std::string bareTempInstallDirectory =
     this->GetOption("CPACK_TEMPORARY_DIRECTORY");
   std::string tempInstallDirectory = bareTempInstallDirectory;
-  bool setDestDir = this->GetOption("CPACK_SET_DESTDIR").IsOn() ||
-    cmIsInternallyOn(this->GetOption("CPACK_SET_DESTDIR"));
+  cmValue v = this->GetOption("CPACK_SET_DESTDIR");
+  bool setDestDir = v.IsOn() || cmIsInternallyOn(v);
   if (!setDestDir) {
     tempInstallDirectory += this->GetPackagingInstallPrefix();
   }
@@ -229,7 +256,7 @@ int cmCPackGenerator::InstallProject()
     this->GetOption("CPACK_INSTALL_DEFAULT_DIRECTORY_PERMISSIONS");
   if (cmNonempty(default_dir_install_permissions)) {
     cmList items{ default_dir_install_permissions };
-    for (const auto& arg : items) {
+    for (auto const& arg : items) {
       if (!cmFSPermissions::stringToModeT(arg, default_dir_mode_v)) {
         cmCPackLogger(cmCPackLog::LOG_ERROR,
                       "Invalid permission value '"
@@ -277,8 +304,8 @@ int cmCPackGenerator::InstallProject()
   // Run pre-build actions
   cmValue preBuildScripts = this->GetOption("CPACK_PRE_BUILD_SCRIPTS");
   if (preBuildScripts) {
-    const cmList scripts{ preBuildScripts };
-    for (const auto& script : scripts) {
+    cmList const scripts{ preBuildScripts };
+    for (auto const& script : scripts) {
       cmCPackLogger(cmCPackLog::LOG_OUTPUT,
                     "Executing pre-build script: " << script << std::endl);
 
@@ -299,7 +326,7 @@ int cmCPackGenerator::InstallProject()
 }
 
 int cmCPackGenerator::InstallProjectViaInstallCommands(
-  bool setDestDir, const std::string& tempInstallDirectory)
+  bool setDestDir, std::string const& tempInstallDirectory)
 {
   (void)setDestDir;
   cmValue installCommands = this->GetOption("CPACK_INSTALL_COMMANDS");
@@ -335,8 +362,8 @@ int cmCPackGenerator::InstallProjectViaInstallCommands(
 }
 
 int cmCPackGenerator::InstallProjectViaInstalledDirectories(
-  bool setDestDir, const std::string& tempInstallDirectory,
-  const mode_t* default_dir_mode)
+  bool setDestDir, std::string const& tempInstallDirectory,
+  mode_t const* default_dir_mode)
 {
   (void)setDestDir;
   (void)tempInstallDirectory;
@@ -364,7 +391,7 @@ int cmCPackGenerator::InstallProjectViaInstalledDirectories(
       return 0;
     }
     cmList::iterator it;
-    const std::string& tempDir = tempInstallDirectory;
+    std::string const& tempDir = tempInstallDirectory;
     for (it = installDirectoriesList.begin();
          it != installDirectoriesList.end(); ++it) {
       std::vector<std::pair<std::string, std::string>> symlinkedFiles;
@@ -434,10 +461,7 @@ int cmCPackGenerator::InstallProjectViaInstalledDirectories(
         cmWorkingDirectory workdir(goToDir);
         if (workdir.Failed()) {
           cmCPackLogger(cmCPackLog::LOG_ERROR,
-                        "Failed to change working directory to "
-                          << goToDir << " : "
-                          << std::strerror(workdir.GetLastResult())
-                          << std::endl);
+                        workdir.GetError() << std::endl);
           return 0;
         }
         for (auto const& symlinked : symlinkedFiles) {
@@ -475,7 +499,7 @@ int cmCPackGenerator::InstallProjectViaInstalledDirectories(
 }
 
 int cmCPackGenerator::InstallProjectViaInstallScript(
-  bool setDestDir, const std::string& tempInstallDirectory)
+  bool setDestDir, std::string const& tempInstallDirectory)
 {
   cmValue cmakeScripts = this->GetOption("CPACK_INSTALL_SCRIPTS");
   {
@@ -542,8 +566,8 @@ int cmCPackGenerator::InstallProjectViaInstallScript(
 }
 
 int cmCPackGenerator::InstallProjectViaInstallCMakeProjects(
-  bool setDestDir, const std::string& baseTempInstallDirectory,
-  const mode_t* default_dir_mode)
+  bool setDestDir, std::string const& baseTempInstallDirectory,
+  mode_t const* default_dir_mode)
 {
   cmValue cmakeProjects = this->GetOption("CPACK_INSTALL_CMAKE_PROJECTS");
   cmValue cmakeGenerator = this->GetOption("CPACK_CMAKE_GENERATOR");
@@ -684,11 +708,11 @@ int cmCPackGenerator::InstallProjectViaInstallCMakeProjects(
 }
 
 int cmCPackGenerator::RunPreinstallTarget(
-  const std::string& installProjectName, const std::string& installDirectory,
-  cmGlobalGenerator* globalGenerator, const std::string& buildConfig)
+  std::string const& installProjectName, std::string const& installDirectory,
+  cmGlobalGenerator* globalGenerator, std::string const& buildConfig)
 {
   // Does this generator require pre-install?
-  if (const char* preinstall = globalGenerator->GetPreinstallTargetName()) {
+  if (char const* preinstall = globalGenerator->GetPreinstallTargetName()) {
     std::string buildCommand = globalGenerator->GenerateCMakeBuildCommand(
       preinstall, buildConfig, "", "", false);
     cmCPackLogger(cmCPackLog::LOG_DEBUG,
@@ -722,10 +746,10 @@ int cmCPackGenerator::RunPreinstallTarget(
 }
 
 int cmCPackGenerator::InstallCMakeProject(
-  bool setDestDir, const std::string& installDirectory,
-  const std::string& baseTempInstallDirectory, const mode_t* default_dir_mode,
-  const std::string& component, bool componentInstall,
-  const std::string& installSubDirectory, const std::string& buildConfig,
+  bool setDestDir, std::string const& installDirectory,
+  std::string const& baseTempInstallDirectory, mode_t const* default_dir_mode,
+  std::string const& component, bool componentInstall,
+  std::string const& installSubDirectory, std::string const& buildConfig,
   std::string& absoluteDestFiles)
 {
   std::string tempInstallDirectory = baseTempInstallDirectory;
@@ -741,7 +765,7 @@ int cmCPackGenerator::InstallCMakeProject(
   cm.SetHomeOutputDirectory("");
   cm.GetCurrentSnapshot().SetDefaultDefinitions();
   cm.AddCMakePaths();
-  cm.SetProgressCallback([this](const std::string& msg, float prog) {
+  cm.SetProgressCallback([this](std::string const& msg, float prog) {
     this->DisplayVerboseOutput(msg, prog);
   });
   cm.SetTrace(this->Trace);
@@ -967,9 +991,8 @@ int cmCPackGenerator::InstallCMakeProject(
       std::string absoluteDestFileComponent =
         std::string("CPACK_ABSOLUTE_DESTINATION_FILES") + "_" +
         this->GetComponentInstallSuffix(component);
-      if (nullptr != this->GetOption(absoluteDestFileComponent)) {
-        std::string absoluteDestFilesListComponent =
-          cmStrCat(this->GetOption(absoluteDestFileComponent), ';', *d);
+      if (cmValue v = this->GetOption(absoluteDestFileComponent)) {
+        std::string absoluteDestFilesListComponent = cmStrCat(*v, ';', *d);
         this->SetOption(absoluteDestFileComponent,
                         absoluteDestFilesListComponent);
       } else {
@@ -988,7 +1011,7 @@ bool cmCPackGenerator::GenerateChecksumFile(cmCryptoHash& crypto,
                                             cm::string_view filename) const
 {
   std::string packageFileName =
-    cmStrCat(this->GetOption("CPACK_OUTPUT_FILE_PREFIX"), "/", filename);
+    cmStrCat(this->GetOption("CPACK_OUTPUT_FILE_PREFIX"), '/', filename);
   std::string hashFile = cmStrCat(
     packageFileName, "." + cmSystemTools::LowerCase(crypto.GetHashAlgoName()));
   cmsys::ofstream outF(hashFile.c_str());
@@ -1003,11 +1026,11 @@ bool cmCPackGenerator::GenerateChecksumFile(cmCryptoHash& crypto,
   return true;
 }
 
-bool cmCPackGenerator::CopyPackageFile(const std::string& srcFilePath,
+bool cmCPackGenerator::CopyPackageFile(std::string const& srcFilePath,
                                        cm::string_view filename) const
 {
   std::string destFilePath =
-    cmStrCat(this->GetOption("CPACK_OUTPUT_FILE_PREFIX"), "/", filename);
+    cmStrCat(this->GetOption("CPACK_OUTPUT_FILE_PREFIX"), '/', filename);
   cmCPackLogger(cmCPackLog::LOG_DEBUG,
                 "Copy final package(s): "
                   << (!srcFilePath.empty() ? srcFilePath : "(NULL)") << " to "
@@ -1026,7 +1049,7 @@ bool cmCPackGenerator::CopyPackageFile(const std::string& srcFilePath,
   return true;
 }
 
-bool cmCPackGenerator::ReadListFile(const char* moduleName)
+bool cmCPackGenerator::ReadListFile(char const* moduleName)
 {
   bool retval;
   std::string fullPath = this->MakefileMap->GetModulesFile(moduleName);
@@ -1037,7 +1060,7 @@ bool cmCPackGenerator::ReadListFile(const char* moduleName)
 }
 
 template <typename ValueType>
-void cmCPackGenerator::StoreOptionIfNotSet(const std::string& op,
+void cmCPackGenerator::StoreOptionIfNotSet(std::string const& op,
                                            ValueType value)
 {
   cmValue def = this->MakefileMap->GetDefinition(op);
@@ -1047,18 +1070,18 @@ void cmCPackGenerator::StoreOptionIfNotSet(const std::string& op,
   this->StoreOption(op, value);
 }
 
-void cmCPackGenerator::SetOptionIfNotSet(const std::string& op,
-                                         const char* value)
+void cmCPackGenerator::SetOptionIfNotSet(std::string const& op,
+                                         char const* value)
 {
   this->StoreOptionIfNotSet(op, value);
 }
-void cmCPackGenerator::SetOptionIfNotSet(const std::string& op, cmValue value)
+void cmCPackGenerator::SetOptionIfNotSet(std::string const& op, cmValue value)
 {
   this->StoreOptionIfNotSet(op, value);
 }
 
 template <typename ValueType>
-void cmCPackGenerator::StoreOption(const std::string& op, ValueType value)
+void cmCPackGenerator::StoreOption(std::string const& op, ValueType value)
 {
   if (!value) {
     this->MakefileMap->RemoveDefinition(op);
@@ -1070,11 +1093,11 @@ void cmCPackGenerator::StoreOption(const std::string& op, ValueType value)
   this->MakefileMap->AddDefinition(op, value);
 }
 
-void cmCPackGenerator::SetOption(const std::string& op, const char* value)
+void cmCPackGenerator::SetOption(std::string const& op, char const* value)
 {
   this->StoreOption(op, value);
 }
-void cmCPackGenerator::SetOption(const std::string& op, cmValue value)
+void cmCPackGenerator::SetOption(std::string const& op, cmValue value)
 {
   this->StoreOption(op, value);
 }
@@ -1191,8 +1214,8 @@ int cmCPackGenerator::DoPackage()
     this->MakefileMap->AddDefinition(
       "CPACK_PACKAGE_FILES", cmList::to_string(this->packageFileNames));
 
-    const cmList scripts{ postBuildScripts };
-    for (const auto& script : scripts) {
+    cmList const scripts{ postBuildScripts };
+    for (auto const& script : scripts) {
       cmCPackLogger(cmCPackLog::LOG_OUTPUT,
                     "Executing post-build script: " << script << std::endl);
 
@@ -1206,8 +1229,22 @@ int cmCPackGenerator::DoPackage()
   }
 
   /* Prepare checksum algorithm*/
-  cmValue algo = this->GetOption("CPACK_PACKAGE_CHECKSUM");
-  std::unique_ptr<cmCryptoHash> crypto = cmCryptoHash::New(*algo);
+  cmValue algoSignatures = this->GetOption("CPACK_PACKAGE_CHECKSUM");
+  std::vector<std::unique_ptr<cmCryptoHash>> crypto;
+
+  if (cmNonempty(algoSignatures)) {
+    cmList algoList{ algoSignatures };
+    // Keep unique algorithms since generating the same checksum multiple times
+    // is not meaningful.
+    algoList.remove_duplicates();
+    crypto.reserve(algoList.size());
+
+    for (std::string const& algo : algoList) {
+      if (std::unique_ptr<cmCryptoHash> hash = cmCryptoHash::New(algo)) {
+        crypto.push_back(std::move(hash));
+      }
+    }
+  }
 
   /*
    * Copy the generated packages to final destination
@@ -1224,9 +1261,9 @@ int cmCPackGenerator::DoPackage()
     if (!this->CopyPackageFile(pkgFileName, filename)) {
       return 0;
     }
-    /* Generate checksum file */
-    if (crypto) {
-      if (!this->GenerateChecksumFile(*crypto, filename)) {
+    /* Generate checksum files */
+    for (std::unique_ptr<cmCryptoHash> const& hash : crypto) {
+      if (!this->GenerateChecksumFile(*hash, filename)) {
         return 0;
       }
     }
@@ -1235,7 +1272,7 @@ int cmCPackGenerator::DoPackage()
   return 1;
 }
 
-int cmCPackGenerator::Initialize(const std::string& name, cmMakefile* mf)
+int cmCPackGenerator::Initialize(std::string const& name, cmMakefile* mf)
 {
   this->MakefileMap = mf;
   this->Name = name;
@@ -1244,7 +1281,11 @@ int cmCPackGenerator::Initialize(const std::string& name, cmMakefile* mf)
   // Load the project specific config file
   cmValue config = this->GetOption("CPACK_PROJECT_CONFIG_FILE");
   if (config) {
-    mf->ReadListFile(*config);
+    if (!mf->ReadListFile(*config)) {
+      cmCPackLogger(cmCPackLog::LOG_WARNING,
+                    "CPACK_PROJECT_CONFIG_FILE not found: " << *config
+                                                            << std::endl);
+    }
   }
   int result = this->InitializeInternal();
   if (cmSystemTools::GetErrorOccurredFlag()) {
@@ -1266,7 +1307,7 @@ int cmCPackGenerator::Initialize(const std::string& name, cmMakefile* mf)
   if (val1 != val2) {
     // One variable is set but not the other?
     // Then set the other variable to the same value (even if it is invalid).
-    if (val1.Get() != nullptr && val2.Get() == nullptr) {
+    if (val1.Get() && !val2.Get()) {
       cmCPackLogger(cmCPackLog::LOG_WARNING,
                     "Variable CPACK_TEMPORARY_INSTALL_DIRECTORY is set, which "
                     "is not recommended. For backwards-compatibility we will "
@@ -1274,7 +1315,7 @@ int cmCPackGenerator::Initialize(const std::string& name, cmMakefile* mf)
                     "proceed. However, better set neither of them!"
                       << std::endl);
       this->MakefileMap->AddDefinition("CPACK_TEMPORARY_DIRECTORY", val1);
-    } else if (val1.Get() == nullptr && val2.Get() != nullptr) {
+    } else if (!val1.Get() && val2.Get()) {
       cmCPackLogger(
         cmCPackLog::LOG_WARNING,
         "Variable CPACK_TEMPORARY_DIRECTORY is set, which is not recommended."
@@ -1301,7 +1342,7 @@ int cmCPackGenerator::Initialize(const std::string& name, cmMakefile* mf)
                       << std::endl);
       return 0;
     }
-  } else if (val1.Get() != nullptr && val2.Get() != nullptr) {
+  } else if (val1.Get() && val2.Get()) {
     cmCPackLogger(cmCPackLog::LOG_WARNING,
                   "Variables CPACK_TEMPORARY_DIRECTORY and "
                   "CPACK_TEMPORARY_INSTALL_DIRECTORY are both set. Because "
@@ -1318,17 +1359,26 @@ int cmCPackGenerator::InitializeInternal()
   return 1;
 }
 
-bool cmCPackGenerator::IsSet(const std::string& name) const
+bool cmCPackGenerator::IsSet(std::string const& name) const
 {
   return this->MakefileMap->IsSet(name);
 }
 
-bool cmCPackGenerator::IsOn(const std::string& name) const
+cmValue cmCPackGenerator::GetOptionIfSet(std::string const& name) const
+{
+  cmValue ret = this->MakefileMap->GetDefinition(name);
+  if (!ret || ret->empty() || cmIsNOTFOUND(*ret)) {
+    return {};
+  }
+  return ret;
+}
+
+bool cmCPackGenerator::IsOn(std::string const& name) const
 {
   return this->GetOption(name).IsOn();
 }
 
-bool cmCPackGenerator::IsSetToOff(const std::string& op) const
+bool cmCPackGenerator::IsSetToOff(std::string const& op) const
 {
   cmValue ret = this->MakefileMap->GetDefinition(op);
   if (cmNonempty(ret)) {
@@ -1337,7 +1387,7 @@ bool cmCPackGenerator::IsSetToOff(const std::string& op) const
   return false;
 }
 
-bool cmCPackGenerator::IsSetToEmpty(const std::string& op) const
+bool cmCPackGenerator::IsSetToEmpty(std::string const& op) const
 {
   cmValue ret = this->MakefileMap->GetDefinition(op);
   if (ret) {
@@ -1346,7 +1396,7 @@ bool cmCPackGenerator::IsSetToEmpty(const std::string& op) const
   return false;
 }
 
-cmValue cmCPackGenerator::GetOption(const std::string& op) const
+cmValue cmCPackGenerator::GetOption(std::string const& op) const
 {
   cmValue ret = this->MakefileMap->GetDefinition(op);
   if (!ret) {
@@ -1366,7 +1416,7 @@ int cmCPackGenerator::PackageFiles()
   return 0;
 }
 
-const char* cmCPackGenerator::GetInstallPath()
+char const* cmCPackGenerator::GetInstallPath()
 {
   if (!this->InstallPath.empty()) {
     return this->InstallPath.c_str();
@@ -1399,7 +1449,7 @@ const char* cmCPackGenerator::GetInstallPath()
   return this->InstallPath.c_str();
 }
 
-const char* cmCPackGenerator::GetPackagingInstallPrefix()
+char const* cmCPackGenerator::GetPackagingInstallPrefix()
 {
   cmCPackLogger(cmCPackLog::LOG_DEBUG,
                 "GetPackagingInstallPrefix: '"
@@ -1430,15 +1480,15 @@ std::string cmCPackGenerator::FindTemplate(cm::string_view name,
   return ffile;
 }
 
-bool cmCPackGenerator::ConfigureString(const std::string& inString,
+bool cmCPackGenerator::ConfigureString(std::string const& inString,
                                        std::string& outString)
 {
   this->MakefileMap->ConfigureString(inString, outString, true, false);
   return true;
 }
 
-bool cmCPackGenerator::ConfigureFile(const std::string& inName,
-                                     const std::string& outName,
+bool cmCPackGenerator::ConfigureFile(std::string const& inName,
+                                     std::string const& outName,
                                      bool copyOnly /* = false */)
 {
   return this->MakefileMap->ConfigureFile(inName, outName, copyOnly, true,
@@ -1533,7 +1583,7 @@ int cmCPackGenerator::PrepareGroupingKind()
     this->componentPackageMethod = method;
   }
 
-  const char* method_names[] = { "ALL_COMPONENTS_IN_ONE", "IGNORE",
+  char const* method_names[] = { "ALL_COMPONENTS_IN_ONE", "IGNORE",
                                  "ONE_PER_GROUP", "UNKNOWN" };
 
   cmCPackLogger(cmCPackLog::LOG_VERBOSE,
@@ -1546,7 +1596,7 @@ int cmCPackGenerator::PrepareGroupingKind()
 }
 
 std::string cmCPackGenerator::GetSanitizedDirOrFileName(
-  const std::string& name, bool isFullName) const
+  std::string const& name, bool isFullName) const
 {
   if (isFullName) {
 #ifdef _WIN32
@@ -1573,10 +1623,10 @@ std::string cmCPackGenerator::GetSanitizedDirOrFileName(
   }
 
 #ifndef _WIN32
-  constexpr const char* prohibited_chars = "<>\"/\\|?*`";
+  constexpr char const* prohibited_chars = "<>\"/\\|?*`";
 #else
   // Note: Windows also excludes the colon.
-  constexpr const char* prohibited_chars = "<>\"/\\|?*`:";
+  constexpr char const* prohibited_chars = "<>\"/\\|?*`:";
 #endif
   // Given name contains non-supported character?
   // Then return its MD5 hash.
@@ -1590,21 +1640,21 @@ std::string cmCPackGenerator::GetSanitizedDirOrFileName(
 }
 
 std::string cmCPackGenerator::GetComponentInstallSuffix(
-  const std::string& componentName)
+  std::string const& componentName)
 {
   return componentName;
 }
 
 std::string cmCPackGenerator::GetComponentInstallDirNameSuffix(
-  const std::string& componentName)
+  std::string const& componentName)
 {
   return this->GetSanitizedDirOrFileName(
     this->GetComponentInstallSuffix(componentName));
 }
 
 std::string cmCPackGenerator::GetComponentPackageFileName(
-  const std::string& initialPackageFileName,
-  const std::string& groupOrComponentName, bool isGroupName)
+  std::string const& initialPackageFileName,
+  std::string const& groupOrComponentName, bool isGroupName)
 {
 
   /*
@@ -1663,7 +1713,7 @@ bool cmCPackGenerator::WantsComponentInstallation() const
 }
 
 cmCPackInstallationType* cmCPackGenerator::GetInstallationType(
-  const std::string& projectName, const std::string& name)
+  std::string const& projectName, std::string const& name)
 {
   (void)projectName;
   bool hasInstallationType = this->InstallationTypes.count(name) != 0;
@@ -1687,7 +1737,7 @@ cmCPackInstallationType* cmCPackGenerator::GetInstallationType(
 }
 
 cmCPackComponent* cmCPackGenerator::GetComponent(
-  const std::string& projectName, const std::string& name)
+  std::string const& projectName, std::string const& name)
 {
   bool hasComponent = this->Components.count(name) != 0;
   cmCPackComponent* component = &this->Components[name];
@@ -1756,7 +1806,7 @@ cmCPackComponent* cmCPackGenerator::GetComponent(
 }
 
 cmCPackComponentGroup* cmCPackGenerator::GetComponentGroup(
-  const std::string& projectName, const std::string& name)
+  std::string const& projectName, std::string const& name)
 {
   (void)projectName;
   std::string macroPrefix =
