@@ -1,18 +1,18 @@
 /* Distributed under the OSI-approved BSD 3-Clause License.  See accompanying
-   file Copyright.txt or https://cmake.org/licensing for details.  */
-#ifndef cmCTestCVS_h
-#define cmCTestCVS_h
+   file LICENSE.rst or https://cmake.org/licensing for details.  */
+#pragma once
 
 #include "cmConfigure.h" // IWYU pragma: keep
 
-#include "cmCTestVC.h"
-
 #include <iosfwd>
 #include <map>
 #include <string>
 #include <vector>
 
+#include "cmCTestVC.h"
+
 class cmCTest;
+class cmMakefile;
 class cmXMLWriter;
 
 /** \class cmCTestCVS
@@ -23,7 +23,7 @@ class cmCTestCVS : public cmCTestVC
 {
 public:
   /** Construct with a CTest instance and update log stream.  */
-  cmCTestCVS(cmCTest* ctest, std::ostream& log);
+  cmCTestCVS(cmCTest* ctest, cmMakefile* mf, std::ostream& log);
 
   ~cmCTestCVS() override;
 
@@ -39,7 +39,7 @@ private:
   std::map<std::string, Directory> Dirs;
 
   std::string ComputeBranchFlag(std::string const& dir);
-  void LoadRevisions(std::string const& file, const char* branchFlag,
+  void LoadRevisions(std::string const& file, char const* branchFlag,
                      std::vector<Revision>& revisions);
   void WriteXMLDirectory(cmXMLWriter& xml, std::string const& path,
                          Directory const& dir);
@@ -51,5 +51,3 @@ private:
   friend class LogParser;
   friend class UpdateParser;
 };
-
-#endif
