@@ -5,12 +5,9 @@
 #include "cmConfigure.h" // IWYU pragma: keep
 
 #include <string>
-#include <vector>
 
 #include "cmCTestCommand.h"
 
-class cmExecutionStatus;
-
 class cmCTestReadCustomFilesCommand : public cmCTestCommand
 {
 public:
