@@ -1,7 +1,8 @@
 /* Distributed under the OSI-approved BSD 3-Clause License.  See accompanying
-   file Copyright.txt or https://cmake.org/licensing for details.  */
+   file LICENSE.rst or https://cmake.org/licensing for details.  */
 #include "QCMakePresetItemModel.h"
 
+#include "QCMakeSizeType.h"
 #include <QFont>
 
 QCMakePresetItemModel::QCMakePresetItemModel(QObject* parent)
@@ -9,7 +10,7 @@ QCMakePresetItemModel::QCMakePresetItemModel(QObject* parent)
 {
 }
 
-QVariant QCMakePresetItemModel::data(const QModelIndex& index, int role) const
+QVariant QCMakePresetItemModel::data(QModelIndex const& index, int role) const
 {
   switch (role) {
     case Qt::AccessibleDescriptionRole:
@@ -27,7 +28,8 @@ QVariant QCMakePresetItemModel::data(const QModelIndex& index, int role) const
       if (index.internalId() == SEPARATOR_INDEX) {
         return QVariant{};
       }
-      auto const& preset = this->m_presets[index.internalId()];
+      auto const& preset =
+        this->m_presets[static_cast<cm_qsizetype>(index.internalId())];
       return preset.displayName.isEmpty() ? preset.name : preset.displayName;
     }
     case Qt::ToolTipRole:
@@ -37,7 +39,8 @@ QVariant QCMakePresetItemModel::data(const QModelIndex& index, int role) const
       if (index.internalId() == SEPARATOR_INDEX) {
         return QVariant{};
       }
-      return this->m_presets[index.internalId()].description;
+      return this->m_presets[static_cast<cm_qsizetype>(index.internalId())]
+        .description;
     case Qt::UserRole:
       if (index.internalId() == CUSTOM_INDEX) {
         return QVariant{};
@@ -45,7 +48,8 @@ QVariant QCMakePresetItemModel::data(const QModelIndex& index, int role) const
       if (index.internalId() == SEPARATOR_INDEX) {
         return QVariant{};
       }
-      return QVariant::fromValue(this->m_presets[index.internalId()]);
+      return QVariant::fromValue(
+        this->m_presets[static_cast<cm_qsizetype>(index.internalId())]);
     case Qt::FontRole:
       if (index.internalId() == CUSTOM_INDEX) {
         QFont font;
@@ -58,19 +62,20 @@ QVariant QCMakePresetItemModel::data(const QModelIndex& index, int role) const
   }
 }
 
-Qt::ItemFlags QCMakePresetItemModel::flags(const QModelIndex& index) const
+Qt::ItemFlags QCMakePresetItemModel::flags(QModelIndex const& index) const
 {
   Qt::ItemFlags flags =
     Qt::ItemIsEditable | Qt::ItemIsDragEnabled | Qt::ItemIsDropEnabled;
   if (index.internalId() != SEPARATOR_INDEX &&
       (index.internalId() == CUSTOM_INDEX ||
-       this->m_presets[index.internalId()].enabled)) {
+       this->m_presets[static_cast<cm_qsizetype>(index.internalId())]
+         .enabled)) {
     flags |= Qt::ItemIsSelectable | Qt::ItemIsEnabled;
   }
   return flags;
 }
 
-int QCMakePresetItemModel::rowCount(const QModelIndex& parent) const
+int QCMakePresetItemModel::rowCount(QModelIndex const& parent) const
 {
   if (parent.isValid()) {
     return 0;
@@ -78,10 +83,11 @@ int QCMakePresetItemModel::rowCount(const QModelIndex& parent) const
   if (this->m_presets.empty()) {
     return 1;
   }
-  return this->m_presets.size() + 2;
+  // NOLINTNEXTLINE(readability-redundant-casting)
+  return static_cast<int>(this->m_presets.size() + 2);
 }
 
-int QCMakePresetItemModel::columnCount(const QModelIndex& parent) const
+int QCMakePresetItemModel::columnCount(QModelIndex const& parent) const
 {
   if (parent.isValid()) {
     return 0;
@@ -90,7 +96,7 @@ int QCMakePresetItemModel::columnCount(const QModelIndex& parent) const
 }
 
 QModelIndex QCMakePresetItemModel::index(int row, int column,
-                                         const QModelIndex& parent) const
+                                         QModelIndex const& parent) const
 {
   if (parent.isValid() || column != 0 || row < 0 ||
       row >= this->rowCount(QModelIndex{})) {
@@ -108,7 +114,7 @@ QModelIndex QCMakePresetItemModel::index(int row, int column,
   return this->createIndex(row, column, static_cast<quintptr>(row));
 }
 
-QModelIndex QCMakePresetItemModel::parent(const QModelIndex& /*index*/) const
+QModelIndex QCMakePresetItemModel::parent(QModelIndex const& /*index*/) const
 {
   return QModelIndex{};
 }
@@ -125,7 +131,7 @@ void QCMakePresetItemModel::setPresets(QVector<QCMakePreset> const& presets)
   this->endResetModel();
 }
 
-int QCMakePresetItemModel::presetNameToRow(const QString& name) const
+int QCMakePresetItemModel::presetNameToRow(QString const& name) const
 {
   if (this->m_presets.empty()) {
     return 0;
@@ -139,5 +145,6 @@ int QCMakePresetItemModel::presetNameToRow(const QString& name) const
     index++;
   }
 
-  return this->m_presets.size() + 1;
+  // NOLINTNEXTLINE(readability-redundant-casting)
+  return static_cast<int>(this->m_presets.size() + 1);
 }
