@@ -6,7 +6,6 @@
 #include <utility>
 
 #include <cm/memory>
-#include <cm/string_view>
 
 #include "cmFileSet.h"
 #include "cmGenExContext.h"
@@ -24,8 +23,6 @@
 #include "cmTarget.h"
 #include "cmValue.h"
 
-class cmTargetExport;
-
 cmExportTryCompileFileGenerator::cmExportTryCompileFileGenerator(
   cmGlobalGenerator* gg, std::vector<std::string> const& targets,
   cmMakefile* mf, std::set<std::string> const& langs)
