@@ -1,5 +1,5 @@
 /* Distributed under the OSI-approved BSD 3-Clause License.  See accompanying
-   file Copyright.txt or https://cmake.org/licensing for details.  */
+   file LICENSE.rst or https://cmake.org/licensing for details.  */
 #include "cmExtraSublimeTextGenerator.h"
 
 #include <cstring>
@@ -14,6 +14,7 @@
 #include "cmGeneratorExpression.h"
 #include "cmGeneratorTarget.h"
 #include "cmGlobalGenerator.h"
+#include "cmList.h"
 #include "cmLocalGenerator.h"
 #include "cmMakefile.h"
 #include "cmMessageType.h"
@@ -43,7 +44,8 @@ cmExtraSublimeTextGenerator::GetFactory()
 {
   static cmExternalMakefileProjectGeneratorSimpleFactory<
     cmExtraSublimeTextGenerator>
-    factory("Sublime Text 2", "Generates Sublime Text 2 project files.");
+    factory("Sublime Text 2",
+            "Generates Sublime Text 2 project files (deprecated).");
 
   if (factory.GetSupportedGlobalGenerators().empty()) {
 #if defined(_WIN32)
@@ -79,35 +81,35 @@ void cmExtraSublimeTextGenerator::Generate()
 }
 
 void cmExtraSublimeTextGenerator::CreateProjectFile(
-  const std::vector<cmLocalGenerator*>& lgs)
+  std::vector<cmLocalGenerator*> const& lgs)
 {
   std::string outputDir = lgs[0]->GetCurrentBinaryDirectory();
   std::string projectName = lgs[0]->GetProjectName();
 
-  const std::string filename =
+  std::string const filename =
     outputDir + "/" + projectName + ".sublime-project";
 
   this->CreateNewProjectFile(lgs, filename);
 }
 
 void cmExtraSublimeTextGenerator::CreateNewProjectFile(
-  const std::vector<cmLocalGenerator*>& lgs, const std::string& filename)
+  std::vector<cmLocalGenerator*> const& lgs, std::string const& filename)
 {
-  const cmMakefile* mf = lgs[0]->GetMakefile();
+  cmMakefile const* mf = lgs[0]->GetMakefile();
 
   cmGeneratedFileStream fout(filename);
   if (!fout) {
     return;
   }
 
-  const std::string& sourceRootRelativeToOutput = cmSystemTools::RelativePath(
+  std::string const& sourceRootRelativeToOutput = cmSystemTools::RelativePath(
     lgs[0]->GetBinaryDirectory(), lgs[0]->GetSourceDirectory());
   // Write the folder entries to the project file
   fout << "{\n";
   fout << "\t\"folders\":\n\t[\n\t";
   if (!sourceRootRelativeToOutput.empty()) {
     fout << "\t{\n\t\t\t\"path\": \"" << sourceRootRelativeToOutput << "\"";
-    const std::string& outputRelativeToSourceRoot =
+    std::string const& outputRelativeToSourceRoot =
       cmSystemTools::RelativePath(lgs[0]->GetSourceDirectory(),
                                   lgs[0]->GetBinaryDirectory());
     if ((!outputRelativeToSourceRoot.empty()) &&
@@ -137,7 +139,7 @@ void cmExtraSublimeTextGenerator::CreateNewProjectFile(
   // End of build_systems
   fout << "\n\t]";
   std::string systemName = mf->GetSafeDefinition("CMAKE_SYSTEM_NAME");
-  std::vector<std::string> tokens = cmExpandedList(this->EnvSettings);
+  cmList tokens{ this->EnvSettings };
 
   if (!this->EnvSettings.empty()) {
     fout << ",";
@@ -168,10 +170,10 @@ void cmExtraSublimeTextGenerator::CreateNewProjectFile(
 }
 
 void cmExtraSublimeTextGenerator::AppendAllTargets(
-  const std::vector<cmLocalGenerator*>& lgs, const cmMakefile* mf,
+  std::vector<cmLocalGenerator*> const& lgs, cmMakefile const* mf,
   cmGeneratedFileStream& fout, MapSourceFileFlags& sourceFileFlags)
 {
-  const std::string& make = mf->GetRequiredDefinition("CMAKE_MAKE_PROGRAM");
+  std::string const& make = mf->GetRequiredDefinition("CMAKE_MAKE_PROGRAM");
   std::string compiler;
   if (!lgs.empty()) {
     this->AppendTarget(fout, "all", lgs[0], nullptr, make.c_str(), mf,
@@ -184,8 +186,8 @@ void cmExtraSublimeTextGenerator::AppendAllTargets(
   // and UTILITY targets
   for (cmLocalGenerator* lg : lgs) {
     cmMakefile* makefile = lg->GetMakefile();
-    const auto& targets = lg->GetGeneratorTargets();
-    for (const auto& target : targets) {
+    auto const& targets = lg->GetGeneratorTargets();
+    for (auto const& target : targets) {
       std::string targetName = target->GetName();
       switch (target->GetType()) {
         case cmStateEnums::GLOBAL_TARGET: {
@@ -234,13 +236,13 @@ void cmExtraSublimeTextGenerator::AppendAllTargets(
 }
 
 void cmExtraSublimeTextGenerator::AppendTarget(
-  cmGeneratedFileStream& fout, const std::string& targetName,
-  cmLocalGenerator* lg, cmGeneratorTarget* target, const char* make,
-  const cmMakefile* makefile, const char* /*compiler*/,
+  cmGeneratedFileStream& fout, std::string const& targetName,
+  cmLocalGenerator* lg, cmGeneratorTarget* target, char const* make,
+  cmMakefile const* makefile, char const* /*compiler*/,
   MapSourceFileFlags& sourceFileFlags, bool firstTarget)
 {
 
-  if (target != nullptr) {
+  if (target) {
     std::vector<cmSourceFile*> sourceFiles;
     target->GetSourceFiles(sourceFiles,
                            makefile->GetSafeDefinition("CMAKE_BUILD_TYPE"));
@@ -264,11 +266,11 @@ void cmExtraSublimeTextGenerator::AppendTarget(
       cmsys::RegularExpression flagRegex;
       // Regular expression to extract compiler flags from a string
       // https://gist.github.com/3944250
-      const char* regexString =
+      char const* regexString =
         R"((^|[ ])-[DIOUWfgs][^= ]+(=\"[^"]+\"|=[^"][^ ]+)?)";
       flagRegex.compile(regexString);
       std::string workString =
-        cmStrCat(flagsString, " ", definesString, " ", includesString);
+        cmStrCat(flagsString, ' ', definesString, ' ', includesString);
       while (flagRegex.find(workString)) {
         std::string::size_type start = flagRegex.start();
         if (workString[start] == ' ') {
@@ -309,8 +311,8 @@ void cmExtraSublimeTextGenerator::AppendTarget(
 // Create the command line for building the given target using the selected
 // make
 std::string cmExtraSublimeTextGenerator::BuildMakeCommand(
-  const std::string& make, const std::string& makefile,
-  const std::string& target)
+  std::string const& make, std::string const& makefile,
+  std::string const& target)
 {
   std::string command = cmStrCat('"', make, '"');
   std::string generator = this->GlobalGenerator->GetName();
@@ -363,12 +365,12 @@ std::string cmExtraSublimeTextGenerator::ComputeFlagsForObject(
   cmGeneratorExpressionInterpreter genexInterpreter(lg, config, gtgt,
                                                     language);
 
-  const std::string COMPILE_FLAGS("COMPILE_FLAGS");
+  std::string const COMPILE_FLAGS("COMPILE_FLAGS");
   if (cmValue cflags = source->GetProperty(COMPILE_FLAGS)) {
     lg->AppendFlags(flags, genexInterpreter.Evaluate(*cflags, COMPILE_FLAGS));
   }
 
-  const std::string COMPILE_OPTIONS("COMPILE_OPTIONS");
+  std::string const COMPILE_OPTIONS("COMPILE_OPTIONS");
   if (cmValue coptions = source->GetProperty(COMPILE_OPTIONS)) {
     lg->AppendCompileOptions(
       flags, genexInterpreter.Evaluate(*coptions, COMPILE_OPTIONS));
@@ -385,14 +387,14 @@ std::string cmExtraSublimeTextGenerator::ComputeDefines(
 {
   std::set<std::string> defines;
   cmMakefile* makefile = lg->GetMakefile();
-  const std::string& language = source->GetOrDetermineLanguage();
-  const std::string& config = makefile->GetSafeDefinition("CMAKE_BUILD_TYPE");
+  std::string const& language = source->GetOrDetermineLanguage();
+  std::string const& config = makefile->GetSafeDefinition("CMAKE_BUILD_TYPE");
   cmGeneratorExpressionInterpreter genexInterpreter(lg, config, target,
                                                     language);
 
   // Add preprocessor definitions for this target and configuration.
   lg->GetTargetDefines(target, config, language, defines);
-  const std::string COMPILE_DEFINITIONS("COMPILE_DEFINITIONS");
+  std::string const COMPILE_DEFINITIONS("COMPILE_DEFINITIONS");
   if (cmValue compile_defs = source->GetProperty(COMPILE_DEFINITIONS)) {
     lg->AppendDefines(
       defines, genexInterpreter.Evaluate(*compile_defs, COMPILE_DEFINITIONS));
@@ -418,13 +420,13 @@ std::string cmExtraSublimeTextGenerator::ComputeIncludes(
 {
   std::vector<std::string> includes;
   cmMakefile* makefile = lg->GetMakefile();
-  const std::string& language = source->GetOrDetermineLanguage();
-  const std::string& config = makefile->GetSafeDefinition("CMAKE_BUILD_TYPE");
+  std::string const& language = source->GetOrDetermineLanguage();
+  std::string const& config = makefile->GetSafeDefinition("CMAKE_BUILD_TYPE");
   cmGeneratorExpressionInterpreter genexInterpreter(lg, config, target,
                                                     language);
 
   // Add include directories for this source file
-  const std::string INCLUDE_DIRECTORIES("INCLUDE_DIRECTORIES");
+  std::string const INCLUDE_DIRECTORIES("INCLUDE_DIRECTORIES");
   if (cmValue cincludes = source->GetProperty(INCLUDE_DIRECTORIES)) {
     lg->AppendIncludeDirectories(
       includes, genexInterpreter.Evaluate(*cincludes, INCLUDE_DIRECTORIES),
@@ -440,8 +442,8 @@ std::string cmExtraSublimeTextGenerator::ComputeIncludes(
   return includesString;
 }
 
-bool cmExtraSublimeTextGenerator::Open(const std::string& bindir,
-                                       const std::string& projectName,
+bool cmExtraSublimeTextGenerator::Open(std::string const& bindir,
+                                       std::string const& projectName,
                                        bool dryRun)
 {
   cmValue sublExecutable =
