@@ -1,14 +1,13 @@
 /* Distributed under the OSI-approved BSD 3-Clause License.  See accompanying
-   file Copyright.txt or https://cmake.org/licensing for details.  */
-#ifndef cmFileInstaller_h
-#define cmFileInstaller_h
+   file LICENSE.rst or https://cmake.org/licensing for details.  */
+#pragma once
 
 #include "cmConfigure.h" // IWYU pragma: keep
 
 #include <string>
-#include <vector>
 
 #include "cmFileCopier.h"
+#include "cmInstallMode.h"
 #include "cmInstallType.h"
 
 class cmExecutionStatus;
@@ -19,12 +18,13 @@ struct cmFileInstaller : public cmFileCopier
   ~cmFileInstaller() override;
 
 protected:
-  cmInstallType InstallType;
-  bool Optional;
-  bool MessageAlways;
-  bool MessageLazy;
-  bool MessageNever;
-  int DestDirLength;
+  cmInstallType InstallType = cmInstallType_FILES;
+  cmInstallMode InstallMode = cmInstallMode::COPY;
+  bool Optional = false;
+  bool MessageAlways = false;
+  bool MessageLazy = false;
+  bool MessageNever = false;
+  int DestDirLength = 0;
   std::string Rename;
 
   std::string Manifest;
@@ -32,11 +32,12 @@ protected:
 
   std::string const& ToName(std::string const& fromName) override;
 
-  void ReportCopy(const std::string& toFile, Type type, bool copy) override;
-  bool ReportMissing(const std::string& fromFile) override;
-  bool Install(const std::string& fromFile,
-               const std::string& toFile) override;
-
+  void ReportCopy(std::string const& toFile, Type type, bool copy) override;
+  bool ReportMissing(std::string const& fromFile) override;
+  bool Install(std::string const& fromFile,
+               std::string const& toFile) override;
+  bool InstallFile(std::string const& fromFile, std::string const& toFile,
+                   MatchProperties match_properties) override;
   bool Parse(std::vector<std::string> const& args) override;
   enum
   {
@@ -47,8 +48,6 @@ protected:
   bool CheckKeyword(std::string const& arg) override;
   bool CheckValue(std::string const& arg) override;
   void DefaultFilePermissions() override;
-  bool GetTargetTypeFromString(const std::string& stype);
+  bool GetTargetTypeFromString(std::string const& stype);
   bool HandleInstallDestination();
 };
-
-#endif
