@@ -1,5 +1,5 @@
 /* Distributed under the OSI-approved BSD 3-Clause License.  See accompanying
-   file Copyright.txt or https://cmake.org/licensing for details.  */
+   file LICENSE.rst or https://cmake.org/licensing for details.  */
 #pragma once
 
 #include "cmConfigure.h" // IWYU pragma: keep
@@ -7,7 +7,8 @@
 #include <string>
 
 #if defined(_WIN32)
-#  include <windows.h> // HANDLE
+#  include <memory>
+using HANDLE = void*;
 #endif
 
 class cmFileLockResult;
@@ -33,7 +34,7 @@ public:
    * @brief Lock the file.
    * @param timeoutSec Lock timeout. If -1 try until success or fatal error.
    */
-  cmFileLockResult Lock(const std::string& filename, unsigned long timeoutSec);
+  cmFileLockResult Lock(std::string const& filename, unsigned long timeoutSec);
 
   /**
    * @brief Unlock the file.
@@ -45,7 +46,7 @@ public:
    * @details This function helps to find double locks (deadlocks) and to do
    * explicit unlocks.
    */
-  bool IsLocked(const std::string& filename) const;
+  bool IsLocked(std::string const& filename) const;
 
 private:
   cmFileLockResult OpenFile();
@@ -53,8 +54,9 @@ private:
   cmFileLockResult LockWithTimeout(unsigned long timeoutSec);
 
 #if defined(_WIN32)
-  HANDLE File = INVALID_HANDLE_VALUE;
-  BOOL LockFile(DWORD flags);
+  HANDLE File = (HANDLE)-1;
+  std::unique_ptr<struct _OVERLAPPED> Overlapped;
+  int LockFile(int flags);
 #else
   int File = -1;
   int LockFile(int cmd, int type) const;
