@@ -23,7 +23,7 @@ class cmFindBase : public cmFindCommon
 {
 public:
   cmFindBase(std::string findCommandName, cmExecutionStatus& status);
-  virtual ~cmFindBase() = default;
+  ~cmFindBase() override;
 
   /**
    * This is called when the command is first encountered in
@@ -40,10 +40,8 @@ protected:
   friend class cmFindBaseDebugState;
   void ExpandPaths();
 
-  // see if the VariableName is already set,
-  // also copy the documentation from the cache to VariableDocumentation
-  // if it has documentation in the cache
-  bool CheckForVariableDefined();
+  bool IsFound() const override;
+  bool IsDefined() const override;
 
   void NormalizeFindResult();
   void StoreFindResult(std::string const& value);
@@ -62,7 +60,6 @@ protected:
   // CMAKE_*_PATH CMAKE_SYSTEM_*_PATH FRAMEWORK|LIBRARY|INCLUDE|PROGRAM
   std::string EnvironmentPath; // LIB,INCLUDE
 
-  bool AlreadyDefined = false;
   bool AlreadyInCacheWithoutMetaInfo = false;
   bool StoreResultInCache = true;
 
@@ -71,6 +68,18 @@ protected:
   std::string ValidatorName;
 
 private:
+  enum class FindState
+  {
+    Undefined,
+    Found,
+    NotFound,
+  };
+  // see if the VariableName is already set,
+  // also copy the documentation from the cache to VariableDocumentation
+  // if it has documentation in the cache
+  FindState GetInitialState();
+  FindState InitialState = FindState::Undefined;
+
   // Add pieces of the search.
   void FillPackageRootPath();
   void FillCMakeVariablePath();
@@ -81,17 +90,16 @@ private:
   void FillUserGuessPath();
 };
 
-class cmFindBaseDebugState
+class cmFindBaseDebugState : public cmFindCommonDebugState
 {
 public:
-  explicit cmFindBaseDebugState(std::string name, cmFindBase const* findBase);
-  ~cmFindBaseDebugState();
-
-  void FoundAt(std::string const& path, std::string regexName = std::string());
-  void FailedAt(std::string const& path,
-                std::string regexName = std::string());
+  explicit cmFindBaseDebugState(cmFindBase const* findBase);
+  ~cmFindBaseDebugState() override;
 
 private:
+  void FoundAtImpl(std::string const& path, std::string regexName) override;
+  void FailedAtImpl(std::string const& path, std::string regexName) override;
+
   struct DebugLibState
   {
     DebugLibState() = default;
@@ -104,8 +112,14 @@ private:
     std::string path;
   };
 
-  cmFindBase const* FindCommand;
-  std::string CommandName;
+  void WriteDebug() const override;
+#ifndef CMAKE_BOOTSTRAP
+  void WriteEvent(cmConfigureLog& log, cmMakefile const& mf) const override;
+  std::vector<std::pair<VariableSource, std::string>> ExtraSearchVariables()
+    const override;
+#endif
+
+  cmFindBase const* const FindBaseCommand;
   std::vector<DebugLibState> FailedSearchLocations;
   DebugLibState FoundSearchLocation;
 };
