@@ -1,5 +1,5 @@
 /* Distributed under the OSI-approved BSD 3-Clause License.  See accompanying
-   file Copyright.txt or https://cmake.org/licensing for details.  */
+   file LICENSE.rst or https://cmake.org/licensing for details.  */
 #include "cmFunctionBlocker.h"
 
 #include <cassert>
@@ -12,29 +12,39 @@
 #include "cmMakefile.h"
 #include "cmMessageType.h"
 
-bool cmFunctionBlocker::IsFunctionBlocked(const cmListFileFunction& lff,
+bool cmFunctionBlocker::IsFunctionBlocked(cmListFileFunction const& lff,
                                           cmExecutionStatus& status)
 {
-  if (lff.Name.Lower == this->StartCommandName()) {
+  if (lff.LowerCaseName() == this->StartCommandName()) {
     this->ScopeDepth++;
-  } else if (lff.Name.Lower == this->EndCommandName()) {
+  } else if (lff.LowerCaseName() == this->EndCommandName()) {
     this->ScopeDepth--;
     if (this->ScopeDepth == 0U) {
       cmMakefile& mf = status.GetMakefile();
       auto self = mf.RemoveFunctionBlocker();
       assert(self.get() == this);
 
-      if (!this->ArgumentsMatch(lff, mf)) {
-        cmListFileContext const& lfc = this->GetStartingContext();
-        cmListFileContext closingContext =
-          cmListFileContext::FromCommandContext(lff, lfc.FilePath);
+      cmListFileContext const& lfc = this->GetStartingContext();
+      cmListFileContext closingContext =
+        cmListFileContext::FromListFileFunction(lff, lfc.FilePath);
+      if (this->EndCommandSupportsArguments() &&
+          !this->ArgumentsMatch(lff, mf)) {
         std::ostringstream e;
         /* clang-format off */
         e << "A logical block opening on the line\n"
           << "  " << lfc << "\n"
           << "closes on the line\n"
           << "  " << closingContext << "\n"
-          << "with mis-matching arguments.";
+          << "with mis-matching arguments.";  // noqa: spellcheck disable-line
+        /* clang-format on */
+        mf.IssueMessage(MessageType::AUTHOR_WARNING, e.str());
+      } else if (!this->EndCommandSupportsArguments() &&
+                 !lff.Arguments().empty()) {
+        std::ostringstream e;
+        /* clang-format off */
+        e << "A logical block closing on the line\n"
+          "  " << closingContext << "\n"
+          "has unexpected arguments.";
         /* clang-format on */
         mf.IssueMessage(MessageType::AUTHOR_WARNING, e.str());
       }
