@@ -1,7 +1,6 @@
 /* Distributed under the OSI-approved BSD 3-Clause License.  See accompanying
-   file Copyright.txt or https://cmake.org/licensing for details.  */
-#ifndef cmGccDepfileReaderTypes_h
-#define cmGccDepfileReaderTypes_h
+   file LICENSE.rst or https://cmake.org/licensing for details.  */
+#pragma once
 
 #include <string>
 #include <vector>
@@ -13,5 +12,3 @@ struct cmGccStyleDependency
 };
 
 using cmGccDepfileContent = std::vector<cmGccStyleDependency>;
-
-#endif
