@@ -1,15 +1,19 @@
 /* Distributed under the OSI-approved BSD 3-Clause License.  See accompanying
-   file Copyright.txt or https://cmake.org/licensing for details.  */
+   file LICENSE.rst or https://cmake.org/licensing for details.  */
 #include "cmGeneratorExpression.h"
 
 #include <algorithm>
 #include <cassert>
 #include <memory>
+#include <stack>
 #include <utility>
 
+#include <cm/string_view>
+
 #include "cmsys/RegularExpression.hxx"
 
-#include "cmGeneratorExpressionContext.h"
+#include "cmGenExContext.h"
+#include "cmGenExEvaluation.h"
 #include "cmGeneratorExpressionDAGChecker.h"
 #include "cmGeneratorExpressionEvaluator.h"
 #include "cmGeneratorExpressionLexer.h"
@@ -40,7 +44,7 @@ std::unique_ptr<cmCompiledGeneratorExpression> cmGeneratorExpression::Parse(
 }
 
 std::string cmGeneratorExpression::Evaluate(
-  std::string input, cmLocalGenerator* lg, const std::string& config,
+  std::string input, cmLocalGenerator const* lg, std::string const& config,
   cmGeneratorTarget const* headTarget,
   cmGeneratorExpressionDAGChecker* dagChecker,
   cmGeneratorTarget const* currentTarget, std::string const& language)
@@ -51,24 +55,31 @@ std::string cmGeneratorExpression::Evaluate(
       "genex_compile_eval", input);
 #endif
 
+    cm::GenEx::Context context(lg, config, language);
     cmCompiledGeneratorExpression cge(*lg->GetCMakeInstance(),
                                       cmListFileBacktrace(), std::move(input));
-    return cge.Evaluate(lg, config, headTarget, dagChecker, currentTarget,
-                        language);
+    return cge.Evaluate(context, dagChecker, headTarget, currentTarget);
   }
   return input;
 }
 
-const std::string& cmCompiledGeneratorExpression::Evaluate(
-  cmLocalGenerator* lg, const std::string& config,
-  const cmGeneratorTarget* headTarget,
-  cmGeneratorExpressionDAGChecker* dagChecker,
-  const cmGeneratorTarget* currentTarget, std::string const& language) const
+std::string const& cmCompiledGeneratorExpression::Evaluate(
+  cmLocalGenerator const* lg, std::string const& config,
+  cmGeneratorTarget const* headTarget) const
 {
-  cmGeneratorExpressionContext context(
-    lg, config, this->Quiet, headTarget,
-    currentTarget ? currentTarget : headTarget, this->EvaluateForBuildsystem,
-    this->Backtrace, language);
+  cm::GenEx::Context context(lg, config);
+  return this->Evaluate(context, nullptr, headTarget);
+}
+
+std::string const& cmCompiledGeneratorExpression::Evaluate(
+  cm::GenEx::Context const& context,
+  cmGeneratorExpressionDAGChecker* dagChecker,
+  cmGeneratorTarget const* headTarget,
+  cmGeneratorTarget const* currentTarget) const
+{
+  cm::GenEx::Evaluation eval(context, this->Quiet, headTarget,
+                             currentTarget ? currentTarget : headTarget,
+                             this->EvaluateForBuildsystem, this->Backtrace);
 
   if (!this->NeedsEvaluation) {
     return this->Input;
@@ -76,29 +87,29 @@ const std::string& cmCompiledGeneratorExpression::Evaluate(
 
   this->Output.clear();
 
-  for (const auto& it : this->Evaluators) {
-    this->Output += it->Evaluate(&context, dagChecker);
+  for (auto const& it : this->Evaluators) {
+    this->Output += it->Evaluate(&eval, dagChecker);
 
-    this->SeenTargetProperties.insert(context.SeenTargetProperties.cbegin(),
-                                      context.SeenTargetProperties.cend());
-    if (context.HadError) {
+    this->SeenTargetProperties.insert(eval.SeenTargetProperties.cbegin(),
+                                      eval.SeenTargetProperties.cend());
+    if (eval.HadError) {
       this->Output.clear();
       break;
     }
   }
 
-  this->MaxLanguageStandard = context.MaxLanguageStandard;
+  this->MaxLanguageStandard = eval.MaxLanguageStandard;
 
-  if (!context.HadError) {
-    this->HadContextSensitiveCondition = context.HadContextSensitiveCondition;
-    this->HadHeadSensitiveCondition = context.HadHeadSensitiveCondition;
+  if (!eval.HadError) {
+    this->HadContextSensitiveCondition = eval.HadContextSensitiveCondition;
+    this->HadHeadSensitiveCondition = eval.HadHeadSensitiveCondition;
     this->HadLinkLanguageSensitiveCondition =
-      context.HadLinkLanguageSensitiveCondition;
-    this->SourceSensitiveTargets = context.SourceSensitiveTargets;
+      eval.HadLinkLanguageSensitiveCondition;
+    this->SourceSensitiveTargets = eval.SourceSensitiveTargets;
   }
 
-  this->DependTargets = context.DependTargets;
-  this->AllTargetsSeen = context.AllTargets;
+  this->DependTargets = eval.DependTargets;
+  this->AllTargetsSeen = eval.AllTargets;
   return this->Output;
 }
 
@@ -123,7 +134,7 @@ cmCompiledGeneratorExpression::cmCompiledGeneratorExpression(
 }
 
 std::string cmGeneratorExpression::StripEmptyListElements(
-  const std::string& input)
+  std::string const& input)
 {
   if (input.find(';') == std::string::npos) {
     return input;
@@ -131,8 +142,8 @@ std::string cmGeneratorExpression::StripEmptyListElements(
   std::string result;
   result.reserve(input.size());
 
-  const char* c = input.c_str();
-  const char* last = c;
+  char const* c = input.c_str();
+  char const* last = c;
   bool skipSemiColons = true;
   for (; *c; ++c) {
     if (*c == ';') {
@@ -154,50 +165,72 @@ std::string cmGeneratorExpression::StripEmptyListElements(
   return result;
 }
 
-static std::string stripAllGeneratorExpressions(const std::string& input)
+static std::string extractAllGeneratorExpressions(
+  std::string const& input,
+  std::map<std::string, std::vector<std::string>>* collected)
 {
   std::string result;
   std::string::size_type pos = 0;
   std::string::size_type lastPos = pos;
-  int nestingLevel = 0;
+  std::stack<char const*> starts; // indices of "$<"
+  std::stack<char const*> colons; // indices of ":"
   while ((pos = input.find("$<", lastPos)) != std::string::npos) {
     result += input.substr(lastPos, pos - lastPos);
+    starts.push(input.c_str() + pos);
     pos += 2;
-    nestingLevel = 1;
-    const char* c = input.c_str() + pos;
-    const char* const cStart = c;
+    char const* c = input.c_str() + pos;
+    char const* const cStart = c;
     for (; *c; ++c) {
       if (cmGeneratorExpression::StartsWithGeneratorExpression(c)) {
-        ++nestingLevel;
+        starts.push(c);
         ++c;
         continue;
       }
-      if (c[0] == '>') {
-        --nestingLevel;
-        if (nestingLevel == 0) {
+      if (c[0] == ':') {
+        if (colons.size() < starts.size()) {
+          colons.push(c);
+        }
+      } else if (c[0] == '>') {
+        if (!colons.empty() && !starts.empty() &&
+            starts.top() < colons.top()) {
+          if (collected) {
+            (*collected)[std::string(starts.top() + 2, colons.top())]
+              .push_back(std::string(colons.top() + 1, c));
+          }
+          colons.pop();
+        }
+        if (!starts.empty()) {
+          starts.pop();
+        }
+        if (starts.empty()) {
           break;
         }
       }
     }
-    const std::string::size_type traversed = (c - cStart) + 1;
+    std::string::size_type const traversed = (c - cStart) + 1;
     if (!*c) {
       result += "$<" + input.substr(pos, traversed);
     }
     pos += traversed;
     lastPos = pos;
   }
-  if (nestingLevel == 0) {
+  if (starts.empty()) {
     result += input.substr(lastPos);
   }
   return cmGeneratorExpression::StripEmptyListElements(result);
 }
 
-static void prefixItems(const std::string& content, std::string& result,
-                        const std::string& prefix)
+static std::string stripAllGeneratorExpressions(std::string const& input)
+{
+  return extractAllGeneratorExpressions(input, nullptr);
+}
+
+static void prefixItems(std::string const& content, std::string& result,
+                        cm::string_view prefix)
 {
   std::vector<std::string> entries;
   cmGeneratorExpression::Split(content, entries);
-  const char* sep = "";
+  char const* sep = "";
   for (std::string const& e : entries) {
     result += sep;
     sep = ";";
@@ -210,8 +243,8 @@ static void prefixItems(const std::string& content, std::string& result,
 }
 
 static std::string stripExportInterface(
-  const std::string& input, cmGeneratorExpression::PreprocessContext context,
-  bool resolveRelative)
+  std::string const& input, cmGeneratorExpression::PreprocessContext context,
+  cm::string_view importPrefix)
 {
   std::string result;
 
@@ -249,8 +282,8 @@ static std::string stripExportInterface(
       assert(false && "Invalid position found");
     }
     nestingLevel = 1;
-    const char* c = input.c_str() + pos;
-    const char* const cStart = c;
+    char const* c = input.c_str() + pos;
+    char const* const cStart = c;
     for (; *c; ++c) {
       if (cmGeneratorExpression::StartsWithGeneratorExpression(c)) {
         ++nestingLevel;
@@ -267,9 +300,9 @@ static std::string stripExportInterface(
           result += input.substr(pos, c - cStart);
         } else if (context == cmGeneratorExpression::InstallInterface &&
                    foundGenex == FoundGenex::InstallInterface) {
-          const std::string content = input.substr(pos, c - cStart);
-          if (resolveRelative) {
-            prefixItems(content, result, "${_IMPORT_PREFIX}/");
+          std::string const content = input.substr(pos, c - cStart);
+          if (!importPrefix.empty()) {
+            prefixItems(content, result, importPrefix);
           } else {
             result += content;
           }
@@ -277,7 +310,7 @@ static std::string stripExportInterface(
         break;
       }
     }
-    const std::string::size_type traversed = (c - cStart) + 1;
+    std::string::size_type const traversed = (c - cStart) + 1;
     if (!*c) {
       auto remaining = input.substr(pos, traversed);
       switch (foundGenex) {
@@ -302,7 +335,7 @@ static std::string stripExportInterface(
   return cmGeneratorExpression::StripEmptyListElements(result);
 }
 
-void cmGeneratorExpression::Split(const std::string& input,
+void cmGeneratorExpression::Split(std::string const& input,
                                   std::vector<std::string>& output)
 {
   std::string::size_type pos = 0;
@@ -325,8 +358,8 @@ void cmGeneratorExpression::Split(const std::string& input,
     }
     pos += 2;
     int nestingLevel = 1;
-    const char* c = input.c_str() + pos;
-    const char* const cStart = c;
+    char const* c = input.c_str() + pos;
+    char const* const cStart = c;
     for (; *c; ++c) {
       if (cmGeneratorExpression::StartsWithGeneratorExpression(c)) {
         ++nestingLevel;
@@ -347,7 +380,7 @@ void cmGeneratorExpression::Split(const std::string& input,
         break;
       }
     }
-    const std::string::size_type traversed = (c - cStart) + 1;
+    std::string::size_type const traversed = (c - cStart) + 1;
     output.push_back(preGenex + "$<" + input.substr(pos, traversed));
     pos += traversed;
     lastPos = pos;
@@ -357,15 +390,15 @@ void cmGeneratorExpression::Split(const std::string& input,
   }
 }
 
-std::string cmGeneratorExpression::Preprocess(const std::string& input,
+std::string cmGeneratorExpression::Preprocess(std::string const& input,
                                               PreprocessContext context,
-                                              bool resolveRelative)
+                                              cm::string_view importPrefix)
 {
   if (context == StripAllGeneratorExpressions) {
     return stripAllGeneratorExpressions(input);
   }
   if (context == BuildInterface || context == InstallInterface) {
-    return stripExportInterface(input, context, resolveRelative);
+    return stripExportInterface(input, context, importPrefix);
   }
 
   assert(false &&
@@ -373,17 +406,24 @@ std::string cmGeneratorExpression::Preprocess(const std::string& input,
   return std::string();
 }
 
-std::string::size_type cmGeneratorExpression::Find(const std::string& input)
+std::string cmGeneratorExpression::Collect(
+  std::string const& input,
+  std::map<std::string, std::vector<std::string>>& collected)
 {
-  const std::string::size_type openpos = input.find("$<");
-  if (openpos != std::string::npos &&
-      input.find('>', openpos) != std::string::npos) {
-    return openpos;
-  }
-  return std::string::npos;
+  return extractAllGeneratorExpressions(input, &collected);
 }
 
-bool cmGeneratorExpression::IsValidTargetName(const std::string& input)
+cm::string_view::size_type cmGeneratorExpression::Find(cm::string_view input)
+{
+  cm::string_view::size_type const openpos = input.find("$<");
+  if (openpos != cm::string_view::npos &&
+      input.find('>', openpos) != cm::string_view::npos) {
+    return openpos;
+  }
+  return cm::string_view::npos;
+}
+
+bool cmGeneratorExpression::IsValidTargetName(std::string const& input)
 {
   // The ':' is supported to allow use with IMPORTED targets. At least
   // Qt 4 and 5 IMPORTED targets use ':' as the namespace delimiter.
@@ -393,7 +433,7 @@ bool cmGeneratorExpression::IsValidTargetName(const std::string& input)
 }
 
 void cmGeneratorExpression::ReplaceInstallPrefix(
-  std::string& input, const std::string& replacement)
+  std::string& input, std::string const& replacement)
 {
   std::string::size_type pos = 0;
   std::string::size_type lastPos = pos;
@@ -407,7 +447,7 @@ void cmGeneratorExpression::ReplaceInstallPrefix(
 }
 
 void cmCompiledGeneratorExpression::GetMaxLanguageStandard(
-  const cmGeneratorTarget* tgt, std::map<std::string, std::string>& mapping)
+  cmGeneratorTarget const* tgt, std::map<std::string, std::string>& mapping)
 {
   auto it = this->MaxLanguageStandard.find(tgt);
   if (it != this->MaxLanguageStandard.end()) {
@@ -415,19 +455,24 @@ void cmCompiledGeneratorExpression::GetMaxLanguageStandard(
   }
 }
 
-const std::string& cmGeneratorExpressionInterpreter::Evaluate(
-  std::string expression, const std::string& property)
+std::string const& cmGeneratorExpressionInterpreter::Evaluate(
+  std::string expression, std::string const& property)
 {
   this->CompiledGeneratorExpression =
     this->GeneratorExpression.Parse(std::move(expression));
 
-  // Specify COMPILE_OPTIONS to DAGchecker, same semantic as COMPILE_FLAGS
-  cmGeneratorExpressionDAGChecker dagChecker(
-    this->HeadTarget,
-    property == "COMPILE_FLAGS" ? "COMPILE_OPTIONS" : property, nullptr,
-    nullptr, this->LocalGenerator, this->Config);
+  cm::GenEx::Context context(this->LocalGenerator, this->Config,
+                             this->Language);
 
-  return this->CompiledGeneratorExpression->Evaluate(
-    this->LocalGenerator, this->Config, this->HeadTarget, &dagChecker, nullptr,
-    this->Language);
+  // Specify COMPILE_OPTIONS to DAGchecker, same semantic as COMPILE_FLAGS
+  cmGeneratorExpressionDAGChecker dagChecker{
+    this->HeadTarget,
+    property == "COMPILE_FLAGS" ? "COMPILE_OPTIONS" : property,
+    nullptr,
+    nullptr,
+    context,
+  };
+
+  return this->CompiledGeneratorExpression->Evaluate(context, &dagChecker,
+                                                     this->HeadTarget);
 }
