@@ -11,13 +11,15 @@
 #include <utility>
 #include <vector>
 
-#include "cmsys/RegularExpression.hxx"
-
 #include "cmGeneratedFileStream.h"
 #include "cmLinkItem.h"
 #include "cmLinkItemGraphVisitor.h"
 #include "cmStateTypes.h"
 
+namespace cmsys {
+class RegularExpression;
+}
+
 class cmGlobalGenerator;
 
 /** This class implements writing files for graphviz (dot) for graphs
