@@ -1,7 +1,6 @@
 /* Distributed under the OSI-approved BSD 3-Clause License.  See accompanying
-   file Copyright.txt or https://cmake.org/licensing for details.  */
-#ifndef cmIfCommand_h
-#define cmIfCommand_h
+   file LICENSE.rst or https://cmake.org/licensing for details.  */
+#pragma once
 
 #include "cmConfigure.h" // IWYU pragma: keep
 
@@ -13,5 +12,3 @@ struct cmListFileArgument;
 /// Starts an if block
 bool cmIfCommand(std::vector<cmListFileArgument> const& args,
                  cmExecutionStatus& status);
-
-#endif
