@@ -1,19 +1,21 @@
 /* Distributed under the OSI-approved BSD 3-Clause License.  See accompanying
-   file Copyright.txt or https://cmake.org/licensing for details.  */
+   file LICENSE.rst or https://cmake.org/licensing for details.  */
 #include "cmIncludeDirectoryCommand.h"
 
 #include <algorithm>
 #include <set>
 #include <utility>
 
-#include "cmAlgorithms.h"
+#include <cmext/algorithm>
+
 #include "cmExecutionStatus.h"
 #include "cmGeneratorExpression.h"
 #include "cmMakefile.h"
 #include "cmStringAlgorithms.h"
 #include "cmSystemTools.h"
+#include "cmValue.h"
 
-static void GetIncludes(cmMakefile& mf, const std::string& arg,
+static void GetIncludes(cmMakefile& mf, std::string const& arg,
                         std::vector<std::string>& incs);
 static void NormalizeInclude(cmMakefile& mf, std::string& inc);
 
@@ -58,9 +60,9 @@ bool cmIncludeDirectoryCommand(std::vector<std::string> const& args,
     GetIncludes(mf, *i, includes);
 
     if (before) {
-      cmAppend(beforeIncludes, includes);
+      cm::append(beforeIncludes, includes);
     } else {
-      cmAppend(afterIncludes, includes);
+      cm::append(afterIncludes, includes);
     }
     if (system) {
       systemIncludes.insert(includes.begin(), includes.end());
@@ -87,7 +89,7 @@ bool cmIncludeDirectoryCommand(std::vector<std::string> const& args,
 // output from a program and passing it into a command the cleanup doesn't
 // always happen
 //
-static void GetIncludes(cmMakefile& mf, const std::string& arg,
+static void GetIncludes(cmMakefile& mf, std::string const& arg,
                         std::vector<std::string>& incs)
 {
   // break apart any line feed arguments
