@@ -1,5 +1,5 @@
 /* Distributed under the OSI-approved BSD 3-Clause License.  See accompanying
-   file Copyright.txt or https://cmake.org/licensing for details.  */
+   file LICENSE.rst or https://cmake.org/licensing for details.  */
 #pragma once
 
 #include "cmConfigure.h" // IWYU pragma: keep
@@ -8,8 +8,8 @@
 #include <string>
 
 #include "cmInstallGenerator.h"
-#include "cmListFileCache.h"
 
+class cmListFileBacktrace;
 class cmLocalGenerator;
 class cmMakefile;
 
