@@ -1,5 +1,5 @@
 /* Distributed under the OSI-approved BSD 3-Clause License.  See accompanying
-   file Copyright.txt or https://cmake.org/licensing for details.  */
+   file LICENSE.rst or https://cmake.org/licensing for details.  */
 #pragma once
 
 #include "cmConfigure.h" // IWYU pragma: keep
@@ -11,10 +11,8 @@
 #include "cmInstallGenerator.h"
 #include "cmInstallType.h"
 #include "cmListFileCache.h"
-#include "cmScriptGenerator.h"
 
 class cmGeneratorTarget;
-class cmLocalGenerator;
 
 /** \class cmInstallTargetGenerator
  * \brief Generate target installation rules.
@@ -39,8 +37,12 @@ public:
     NamelinkModeSkip
   };
   void SetNamelinkMode(NamelinkModeType mode) { this->NamelinkMode = mode; }
+  void SetImportlinkMode(NamelinkModeType mode)
+  {
+    this->ImportlinkMode = mode;
+  }
 
-  std::string GetInstallFilename(const std::string& config) const;
+  std::string GetInstallFilename(std::string const& config) const;
 
   void GetInstallObjectNames(std::string const& config,
                              std::vector<std::string>& objects) const;
@@ -50,11 +52,12 @@ public:
     NameNormal,
     NameImplib,
     NameSO,
-    NameReal
+    NameReal,
+    NameImplibReal
   };
 
-  static std::string GetInstallFilename(const cmGeneratorTarget* target,
-                                        const std::string& config,
+  static std::string GetInstallFilename(cmGeneratorTarget const* target,
+                                        std::string const& config,
                                         NameType nameType = NameNormal);
 
   bool Compute(cmLocalGenerator* lg) override;
@@ -82,6 +85,7 @@ public:
     std::string ToDir;
 
     NamelinkModeType NamelinkMode = NamelinkModeNone;
+    bool Rename = false;
     bool NoTweak = false;
     bool UseSourcePermissions = false;
     cmInstallType Type = cmInstallType();
@@ -91,36 +95,37 @@ public:
   bool GetOptional() const { return this->Optional; }
 
 protected:
-  void GenerateScriptForConfig(std::ostream& os, const std::string& config,
+  void GenerateScriptForConfig(std::ostream& os, std::string const& config,
                                Indent indent) override;
   void PreReplacementTweaks(std::ostream& os, Indent indent,
-                            const std::string& config,
+                            std::string const& config,
                             std::string const& file);
   void PostReplacementTweaks(std::ostream& os, Indent indent,
-                             const std::string& config,
+                             std::string const& config,
                              std::string const& file);
   void AddInstallNamePatchRule(std::ostream& os, Indent indent,
-                               const std::string& config,
-                               const std::string& toDestDirPath);
+                               std::string const& config,
+                               std::string const& toDestDirPath);
   void AddChrpathPatchRule(std::ostream& os, Indent indent,
-                           const std::string& config,
+                           std::string const& config,
                            std::string const& toDestDirPath);
   void AddRPathCheckRule(std::ostream& os, Indent indent,
-                         const std::string& config,
+                         std::string const& config,
                          std::string const& toDestDirPath);
 
   void AddStripRule(std::ostream& os, Indent indent,
-                    const std::string& toDestDirPath);
+                    std::string const& toDestDirPath);
   void AddRanlibRule(std::ostream& os, Indent indent,
-                     const std::string& toDestDirPath);
+                     std::string const& toDestDirPath);
   void AddUniversalInstallRule(std::ostream& os, Indent indent,
-                               const std::string& toDestDirPath);
-  void IssueCMP0095Warning(const std::string& unescapedRpath);
+                               std::string const& toDestDirPath);
+  void IssueCMP0095Warning(std::string const& unescapedRpath);
 
   std::string const TargetName;
   cmGeneratorTarget* Target = nullptr;
   std::string const FilePermissions;
   NamelinkModeType NamelinkMode;
+  NamelinkModeType ImportlinkMode;
   bool const ImportLibrary;
   bool const Optional;
 };
