@@ -8,6 +8,7 @@
 
 #include "cmGeneratorTarget.h"
 #include "cmLinkItem.h"
+#include "cmListFileCache.h"
 #include "cmMakefile.h"
 
 void cmLinkItemGraphVisitor::VisitItem(cmLinkItem const& item)
@@ -120,12 +121,12 @@ void cmLinkItemGraphVisitor::GetDependencies(cmGeneratorTarget const& target,
     }
   }
 
-  std::vector<cmGeneratorTarget*> objectLibraries;
+  std::vector<BT<cmGeneratorTarget*>> objectLibraries;
   target.GetObjectLibrariesInSources(objectLibraries);
   for (auto const& lib : objectLibraries) {
-    auto const& name = lib->GetName();
+    auto const& name = lib.Value->GetName();
     if (dependencies.find(name) == dependencies.cend()) {
-      auto objectItem = cmLinkItem(lib, false, lib->GetBacktrace());
+      auto objectItem = cmLinkItem(lib.Value, false, lib.Backtrace);
       dependencies[name] = Dependency(DependencyType::Object, objectItem);
     }
   }
