@@ -1,5 +1,5 @@
 /* Distributed under the OSI-approved BSD 3-Clause License.  See accompanying
-   file Copyright.txt or https://cmake.org/licensing for details.  */
+   file LICENSE.rst or https://cmake.org/licensing for details.  */
 #pragma once
 
 #include "cmConfigure.h" // IWYU pragma: keep
@@ -15,8 +15,8 @@ class cmMakefile;
 /** \class cmLocalVisualStudio10Generator
  * \brief Write Visual Studio 10 project files.
  *
- * cmLocalVisualStudio10Generator produces a Visual Studio 10 project
- * file for each target in its directory.
+ * cmLocalVisualStudio10Generator produces a MSBuild project file for each
+ * target in its directory.
  */
 class cmLocalVisualStudio10Generator : public cmLocalVisualStudio7Generator
 {
@@ -24,13 +24,13 @@ public:
   //! Set cache only and recurse to false by default.
   cmLocalVisualStudio10Generator(cmGlobalGenerator* gg, cmMakefile* mf);
 
-  virtual ~cmLocalVisualStudio10Generator();
+  ~cmLocalVisualStudio10Generator() override;
 
-  void ReadAndStoreExternalGUID(const std::string& name,
-                                const char* path) override;
+  void ReadAndStoreExternalGUID(std::string const& name,
+                                char const* path) override;
 
 protected:
-  const char* ReportErrorLabel() const override;
+  char const* ReportErrorLabel() const override;
   bool CustomCommandUseLocal() const override { return true; }
 
 private:
