@@ -1,14 +1,14 @@
 /* Distributed under the OSI-approved BSD 3-Clause License.  See accompanying
-   file Copyright.txt or https://cmake.org/licensing for details.  */
+   file LICENSE.rst or https://cmake.org/licensing for details.  */
 #include "cmXMLSafe.h"
 
+#include <cstdio>
+#include <cstring>
+#include <sstream>
+
 #include "cm_utf8.h"
 
-#include <sstream>
-#include <stdio.h>
-#include <string.h>
-
-cmXMLSafe::cmXMLSafe(const char* s)
+cmXMLSafe::cmXMLSafe(char const* s)
   : Data(s)
   , Size(static_cast<unsigned long>(strlen(s)))
   , DoQuotes(true)
@@ -28,7 +28,7 @@ cmXMLSafe& cmXMLSafe::Quotes(bool b)
   return *this;
 }
 
-std::string cmXMLSafe::str()
+std::string cmXMLSafe::str() const
 {
   std::ostringstream ss;
   ss << *this;
@@ -41,7 +41,7 @@ std::ostream& operator<<(std::ostream& os, cmXMLSafe const& self)
   char const* last = self.Data + self.Size;
   while (first != last) {
     unsigned int ch;
-    if (const char* next = cm_utf8_decode_character(first, last, &ch)) {
+    if (char const* next = cm_utf8_decode_character(first, last, &ch)) {
       // http://www.w3.org/TR/REC-xml/#NT-Char
       if ((ch >= 0x20 && ch <= 0xD7FF) || (ch >= 0xE000 && ch <= 0xFFFD) ||
           (ch >= 0x10000 && ch <= 0x10FFFF) || ch == 0x9 || ch == 0xA ||
@@ -73,7 +73,7 @@ std::ostream& operator<<(std::ostream& os, cmXMLSafe const& self)
       } else {
         // Use a human-readable hex value for this invalid character.
         char buf[16];
-        sprintf(buf, "%X", ch);
+        snprintf(buf, sizeof(buf), "%X", ch);
         os << "[NON-XML-CHAR-0x" << buf << "]";
       }
 
@@ -82,7 +82,7 @@ std::ostream& operator<<(std::ostream& os, cmXMLSafe const& self)
       ch = static_cast<unsigned char>(*first++);
       // Use a human-readable hex value for this invalid byte.
       char buf[16];
-      sprintf(buf, "%X", ch);
+      snprintf(buf, sizeof(buf), "%X", ch);
       os << "[NON-UTF-8-BYTE-0x" << buf << "]";
     }
   }
