@@ -24,6 +24,7 @@
 #include <windows.h>
 
 #include "cmsys/Encoding.hxx"
+#include "cmsys/SystemTools.hxx"
 
 #include "cmStringAlgorithms.h"
 #include "cmSystemTools.h"
@@ -34,7 +35,7 @@ void _setargv()
 {
 }
 
-static void Fatal(const char* msg, ...)
+static void Fatal(char const* msg, ...)
 {
   va_list ap;
   fprintf(stderr, "ninja: FATAL: ");
@@ -49,7 +50,7 @@ static void Fatal(const char* msg, ...)
   ExitProcess(1);
 }
 
-static void usage(const char* msg)
+static void usage(char const* msg)
 {
   Fatal("%s\n\nusage:\n    "
         "cmcldeps "
@@ -72,8 +73,8 @@ static cm::string_view trimLeadingSpace(cm::string_view cmdline)
   return cmdline.substr(i);
 }
 
-static void replaceAll(std::string& str, const std::string& search,
-                       const std::string& repl)
+static void replaceAll(std::string& str, std::string const& search,
+                       std::string const& repl)
 {
   std::string::size_type pos = 0;
   while ((pos = str.find(search, pos)) != std::string::npos) {
@@ -135,7 +136,7 @@ static void escapePath(std::string& path)
   replaceAll(path, " ", "\\ ");
 }
 
-static void outputDepFile(const std::string& dfile, const std::string& objfile,
+static void outputDepFile(std::string const& dfile, std::string const& objfile,
                           std::vector<std::string>& incs)
 {
 
@@ -151,7 +152,7 @@ static void outputDepFile(const std::string& dfile, const std::string& objfile,
   // FIXME should this be fatal or not? delete obj? delete d?
   if (!out)
     return;
-  std::string cwd = cmSystemTools::GetCurrentWorkingDirectory();
+  std::string cwd = cmsys::SystemTools::GetCurrentWorkingDirectory();
   replaceAll(cwd, "/", "\\");
   cwd += "\\";
 
@@ -176,24 +177,9 @@ static void outputDepFile(const std::string& dfile, const std::string& objfile,
   fclose(out);
 }
 
-bool contains(const std::string& str, const std::string& what)
-{
-  return str.find(what) != std::string::npos;
-}
-
-std::string replace(const std::string& str, const std::string& what,
-                    const std::string& replacement)
-{
-  size_t pos = str.find(what);
-  if (pos == std::string::npos)
-    return str;
-  std::string replaced = str;
-  return replaced.replace(pos, what.size(), replacement);
-}
-
-static int process(cm::string_view srcfilename, const std::string& dfile,
-                   const std::string& objfile, const std::string& prefix,
-                   const std::string& cmd, const std::string& dir = "",
+static int process(cm::string_view srcfilename, std::string const& dfile,
+                   std::string const& objfile, std::string const& prefix,
+                   std::string const& cmd, std::string const& dir = "",
                    bool quiet = false)
 {
   std::string output;
@@ -262,21 +248,26 @@ int main()
     srcfilename = srcfilename.substr(pos + 1);
   }
 
-  std::string nol = " /nologo ";
-  std::string show = " /showIncludes ";
-  if (lang == "C" || lang == "CXX") {
-    return process(srcfilename, dfile, objfile, prefix,
-                   binpath + nol + show + rest);
-  } else if (lang == "RC") {
+  if (lang == "RC") {
     // "misuse" cl.exe to get headers from .rc files
 
+    // Make sure there is at most one /nologo option.
+    bool const haveNologo = (rest.find("/nologo ") != std::string::npos ||
+                             rest.find("-nologo ") != std::string::npos);
+    cmSystemTools::ReplaceString(rest, "-nologo ", " ");
+    cmSystemTools::ReplaceString(rest, "/nologo ", " ");
     std::string clrest = rest;
-    // rc: /fo x.dir\x.rc.res  ->  cl: /out:x.dir\x.rc.res.dep.obj
-    clrest = replace(clrest, "/fo ", "/out:");
-    clrest = replace(clrest, "-fo ", "-out:");
-    clrest = replace(clrest, objfile, objfile + ".dep.obj ");
+    if (haveNologo) {
+      rest = "/nologo " + rest;
+    }
 
-    cl = "\"" + cl + "\" /P /DRC_INVOKED /TC ";
+    // rc /fo X.dir\x.rc.res  =>  cl -FoX.dir\x.rc.res.obj
+    // The object will not actually be written.
+    cmSystemTools::ReplaceString(clrest, "/fo ", " ");
+    cmSystemTools::ReplaceString(clrest, "-fo ", " ");
+    cmSystemTools::ReplaceString(clrest, objfile, "-Fo" + objfile + ".obj");
+
+    cl = "\"" + cl + "\" /P /DRC_INVOKED /nologo /showIncludes /TC ";
 
     // call cl in object dir so the .i is generated there
     std::string objdir;
@@ -288,14 +279,15 @@ int main()
     }
 
     // extract dependencies with cl.exe
-    int exit_code = process(srcfilename, dfile, objfile, prefix,
-                            cl + nol + show + clrest, objdir, true);
+    int exit_code =
+      process(srcfilename, dfile, objfile, prefix, cl + clrest, objdir, true);
 
     if (exit_code != 0)
       return exit_code;
 
     // compile rc file with rc.exe
-    return process(srcfilename, "", objfile, prefix, binpath + " " + rest,
+    std::string rc = cmStrCat('"', binpath, '"');
+    return process(srcfilename, "", objfile, prefix, cmStrCat(rc, ' ', rest),
                    std::string(), true);
   }
 
