@@ -97,7 +97,9 @@ char const* const HELP_AVAILABLE_COMMANDS = R"(Available commands:
   copy <file>... destination  - copy files to destination (either file or directory)
   copy_directory <dir>... destination   - copy content of <dir>... directories to 'destination' directory
   copy_directory_if_different <dir>... destination   - copy changed content of <dir>... directories to 'destination' directory
+  copy_directory_if_newer <dir>... destination   - copy newer content of <dir>... directories to 'destination' directory
   copy_if_different <file>... destination  - copy files if it has changed
+  copy_if_newer <file>... destination  - copy files if source is newer than destination
   echo [<string>...]        - displays arguments as text
   echo_append [<string>...] - displays arguments as text but no new line
   env [--unset=NAME ...] [NAME=VALUE ...] [--] <command> [<arg>...]
@@ -778,15 +780,45 @@ int cmcmd::ExecuteCMakeCommand(std::vector<std::string> const& args,
       return return_value;
     }
 
+    // Copy file if newer.
+    if (args[1] == "copy_if_newer" && args.size() > 3) {
+      // If multiple source files specified,
+      // then destination must be directory
+      if ((args.size() > 4) &&
+          (!cmSystemTools::FileIsDirectory(args.back()))) {
+        std::cerr << "Error: Target (for copy_if_newer command) \""
+                  << args.back() << "\" is not a directory.\n";
+        return 1;
+      }
+      // If error occurs we want to continue copying next files.
+      bool return_value = false;
+      for (auto const& arg : cmMakeRange(args).advance(2).retreat(1)) {
+        if (!cmSystemTools::CopyFileIfNewer(arg, args.back())) {
+          std::cerr << "Error copying file (if newer) from \"" << arg
+                    << "\" to \"" << args.back() << "\".\n";
+          return_value = true;
+        }
+      }
+      return return_value;
+    }
+
     // Copy directory contents
     if ((args[1] == "copy_directory" ||
-         args[1] == "copy_directory_if_different") &&
+         args[1] == "copy_directory_if_different" ||
+         args[1] == "copy_directory_if_newer") &&
         args.size() > 3) {
       // If error occurs we want to continue copying next files.
       bool return_value = false;
-      bool const copy_always = (args[1] == "copy_directory");
+
+      cmsys::SystemTools::CopyWhen when = cmsys::SystemTools::CopyWhen::Always;
+      if (args[1] == "copy_directory_if_different") {
+        when = cmsys::SystemTools::CopyWhen::OnlyIfDifferent;
+      } else if (args[1] == "copy_directory_if_newer") {
+        when = cmsys::SystemTools::CopyWhen::OnlyIfNewer;
+      }
+
       for (auto const& arg : cmMakeRange(args).advance(2).retreat(1)) {
-        if (!cmSystemTools::CopyADirectory(arg, args.back(), copy_always)) {
+        if (!cmSystemTools::CopyADirectory(arg, args.back(), when)) {
           std::cerr << "Error copying directory from \"" << arg << "\" to \""
                     << args.back() << "\".\n";
           return_value = true;
@@ -1349,6 +1381,18 @@ int cmcmd::ExecuteCMakeCommand(std::vector<std::string> const& args,
     }
 
     // Internal CMake dependency scanning support.
+    // The format is: -E cmake_fastbuild_check_depends <dummy_file>
+    // <space_separated_list_of_real_outputs>
+    if (args[1] == "cmake_fastbuild_check_depends" && args.size() >= 3) {
+      auto const dummyFile = args[2];
+      for (auto const& arg : cmMakeRange(args).advance(3)) {
+        if (!cmSystemTools::FileExists(arg)) {
+          cmSystemTools::RemoveFile(dummyFile);
+          return 0;
+        }
+      }
+      return 0;
+    }
     if (args[1] == "cmake_depends" && args.size() >= 6) {
       bool const verbose = isCMakeVerbose();
 
@@ -1361,6 +1405,7 @@ int cmcmd::ExecuteCMakeCommand(std::vector<std::string> const& args,
       std::string homeOutDir;
       std::string startOutDir;
       std::string depInfo;
+      std::string targetName;
       bool color = false;
       if (args.size() >= 8) {
         // Full signature:
@@ -1369,6 +1414,7 @@ int cmcmd::ExecuteCMakeCommand(std::vector<std::string> const& args,
         //                    <home-src-dir> <start-src-dir>
         //                    <home-out-dir> <start-out-dir>
         //                    <dep-info> [--color=$(COLOR)]
+        //                    <target-name>
         //
         // All paths are provided.
         gen = args[2];
@@ -1377,10 +1423,19 @@ int cmcmd::ExecuteCMakeCommand(std::vector<std::string> const& args,
         homeOutDir = args[5];
         startOutDir = args[6];
         depInfo = args[7];
+        size_t targetNameIdx = 8;
         if (args.size() >= 9 && cmHasLiteralPrefix(args[8], "--color=")) {
           // Enable or disable color based on the switch value.
+          targetNameIdx = 9;
           color = (args[8].size() == 8 || cmIsOn(args[8].substr(8)));
         }
+        if (args.size() > targetNameIdx) {
+          targetName = args[targetNameIdx];
+        } else {
+          std::string targetDir = cmSystemTools::GetFilenamePath(depInfo);
+          targetDir = cmSystemTools::GetFilenameName(targetDir);
+          targetName = targetDir.substr(0, targetDir.size() - 4);
+        }
       } else {
         // Support older signature for existing makefiles:
         //
@@ -1396,6 +1451,10 @@ int cmcmd::ExecuteCMakeCommand(std::vector<std::string> const& args,
         homeOutDir = args[3];
         startOutDir = args[3];
         depInfo = args[5];
+        // Strip the `.dir` suffix. Old CMake always uses this pattern.
+        std::string targetDir = cmSystemTools::GetFilenamePath(depInfo);
+        targetDir = cmSystemTools::GetFilenameName(targetDir);
+        targetName = targetDir.substr(0, targetDir.size() - 4);
       }
 
       // Create a local generator configured for the directory in
@@ -1421,7 +1480,9 @@ int cmcmd::ExecuteCMakeCommand(std::vector<std::string> const& args,
         lgd->SetRelativePathTop(homeDir, homeOutDir);
 
         // Actually scan dependencies.
-        return lgd->UpdateDependencies(depInfo, verbose, color) ? 0 : 2;
+        return lgd->UpdateDependencies(depInfo, targetName, verbose, color)
+          ? 0
+          : 2;
       }
       return 1;
     }
@@ -2246,7 +2307,9 @@ int cmcmd::VisualStudioLink(std::vector<std::string> const& args, int type,
   std::vector<std::string> expandedArgs;
   for (std::string const& i : args) {
     // check for nmake temporary files
-    if (i[0] == '@' && !cmHasLiteralPrefix(i, "@CMakeFiles")) {
+    if (i[0] == '@' &&
+        !(cmHasLiteralPrefix(i, "@CMakeFiles") ||
+          cmHasLiteralPrefix(i, "@.o/") || cmHasLiteralPrefix(i, "@.o\\"))) {
       cmsys::ifstream fin(i.substr(1).c_str());
       std::string line;
       while (cmSystemTools::GetLineFromStream(fin, line)) {
