@@ -9,11 +9,11 @@
 #include "cmsys/Encoding.hxx"
 
 #include "cmCTest.h"
-#include "cmConsoleBuf.h"
 #include "cmDocumentation.h"
 #include "cmDocumentationEntry.h"
 #include "cmInstrumentation.h"
 #include "cmInstrumentationQuery.h"
+#include "cmStdIoConsole.h"
 #include "cmSystemTools.h"
 
 #include "CTest/cmCTestLaunch.h"
@@ -149,6 +149,7 @@ cmDocumentationEntry const cmDocumentationOptions[] = {
   { "--extra-submit <file>[;<file>]", "Submit extra files to the dashboard." },
   { "--http-header <header>", "Append HTTP header when submitting" },
   { "--schedule-random", "Use a random order for scheduling tests" },
+  { "--schedule-random-seed", "Override seed for random order of tests" },
   { "--submit-index",
     "Submit individual dashboard tests with specific index" },
   { "--timeout <seconds>", "Set the default test timeout." },
@@ -165,11 +166,7 @@ cmDocumentationEntry const cmDocumentationOptions[] = {
 // this is a test driver program for cmCTest.
 int main(int argc, char const* const* argv)
 {
-  cmSystemTools::EnsureStdPipes();
-
-  // Replace streambuf so we can output Unicode to console
-  cmConsoleBuf consoleBuf;
-  consoleBuf.SetUTF8Pipes();
+  cm::StdIo::Console console;
 
   cmsys::Encoding::CommandLineArguments encoding_args =
     cmsys::Encoding::CommandLineArguments::Main(argc, argv);
@@ -197,8 +194,8 @@ int main(int argc, char const* const* argv)
   // Dispatch 'ctest --collect-instrumentation' once given PID finishes
   if (argc == 4 &&
       strcmp(argv[1], "--wait-and-collect-instrumentation") == 0) {
-    return cmInstrumentation(argv[2]).CollectTimingAfterBuild(
-      std::stoi(argv[3]));
+    return cmInstrumentation(argv[2], cmInstrumentation::LoadQueriesAfter::No)
+      .CollectTimingAfterBuild(std::stoi(argv[3]));
   }
 
   // Dispatch 'ctest --collect-instrumentation' mode directly.
