@@ -6,8 +6,10 @@
 #  define DL_EXPORT
 #endif
 
+#include "testDynload.h"
+
 DL_EXPORT int TestDynamicLoaderData = 0;
 
-DL_EXPORT void TestDynamicLoaderSymbolPointer()
+DL_EXPORT void TestDynamicLoaderSymbolPointer(void)
 {
 }
