@@ -3,8 +3,7 @@
 
 int main(int argc, char** argv)
 {
-  std::fstream fout;
-  fout.open("targetoutput.h", std::ios::out);
+  std::ofstream fout("targetoutput.h");
   if (!fout)
     return 1;
   fout << "#define TARGETOUTPUT_DEFINE\n";
