@@ -1,6 +1,5 @@
-
 #include <cstdint>
-#include <initializer_list>
+#include <functional>
 #include <iostream>
 #include <iterator>
 #include <limits>
@@ -14,24 +13,51 @@ namespace {
 
 int failed = 0;
 
+enum class Test : std::uint8_t
+{
+  A,
+  B,
+  C,
+  D,
+  E
+};
+
+using EnumSetTest = cm::enum_set<Test>;
+
+enum class Test2 : std::uint8_t
+{
+  A,
+  B,
+  C,
+  D,
+  E
+};
+
+using EnumSetTest2 = cm::enum_set<Test2, 5>;
+}
+
+CM_ENUM_SET_TRAITS(EnumSetTest)
+CM_ENUM_SET_TRAITS(EnumSetTest2)
+
+namespace {
 void testDeclaration()
 {
   std::cout << "testDeclaration()" << std::endl;
 
   {
-    enum class Test : std::uint8_t
-    {
-      A,
-      B,
-      C,
-      D
-    };
-    cm::enum_set<Test> testSet1;
-    cm::enum_set<Test> testSet2 = Test::A;
-    cm::enum_set<Test> testSet3 = Test::A | Test::C;
-    cm::enum_set<Test> testSet4 = Test::A + Test::C;
-    cm::enum_set<Test> testSet5{ Test::A, Test::C };
-    cm::enum_set<Test> testSet6 = testSet3;
+    static EnumSetTest const testSet1;
+    static EnumSetTest2 const testSet2;
+    static_cast<void>(testSet1);
+    static_cast<void>(testSet2);
+  }
+
+  {
+    EnumSetTest testSet1;
+    EnumSetTest testSet2 = Test::A;
+    EnumSetTest testSet3 = Test::A | Test::C;
+    EnumSetTest testSet4 = Test::A + Test::C;
+    EnumSetTest testSet5{ Test::A, Test::C };
+    EnumSetTest testSet6 = testSet3;
 
     if (!testSet1.empty()) {
       ++failed;
@@ -48,15 +74,50 @@ void testDeclaration()
     }
   }
   {
-    enum class Test : std::uint8_t
-    {
-      A,
-      B,
-      C,
-      D
-    };
-    cm::enum_set<Test> testSet1;
-    cm::enum_set<Test, 4> testSet2;
+    EnumSetTest2 testSet1;
+    EnumSetTest2 testSet2 = Test2::A;
+    EnumSetTest2 testSet3 = Test2::A | Test2::C;
+    EnumSetTest2 testSet4 = Test2::A + Test2::C;
+    EnumSetTest2 testSet5{ Test2::A, Test2::C };
+    EnumSetTest2 testSet6 = testSet3;
+
+    if (!testSet1.empty()) {
+      ++failed;
+    }
+    if (testSet2.size() != 1) {
+      ++failed;
+    }
+    if (testSet3.size() != 2 || testSet4.size() != 2 || testSet5.size() != 2 ||
+        testSet6.size() != 2) {
+      ++failed;
+    }
+    if (testSet3 != testSet4 || testSet4 != testSet5 || testSet5 != testSet6) {
+      ++failed;
+    }
+  }
+  {
+    using LocalEnumSetTest = cm::enum_set<Test>;
+    LocalEnumSetTest testSet1;
+    LocalEnumSetTest testSet2 = Test::A;
+    LocalEnumSetTest testSet3{ Test::A, Test::C };
+    LocalEnumSetTest testSet4 = testSet3;
+
+    if (!testSet1.empty()) {
+      ++failed;
+    }
+    if (testSet2.size() != 1) {
+      ++failed;
+    }
+    if (testSet3.size() != 2 || testSet4.size() != 2) {
+      ++failed;
+    }
+    if (testSet3 != testSet4) {
+      ++failed;
+    }
+  }
+  {
+    EnumSetTest testSet1;
+    EnumSetTest2 testSet2;
 
     if (testSet1.size() != 0 ||
         testSet1.max_size() !=
@@ -64,7 +125,7 @@ void testDeclaration()
             typename std::underlying_type<Test>::type>::digits) {
       ++failed;
     }
-    if (testSet2.size() != 0 || testSet2.max_size() != 4) {
+    if (testSet2.size() != 0 || testSet2.max_size() != 5) {
       ++failed;
     }
   }
@@ -74,14 +135,7 @@ void testIteration()
 {
   std::cout << "testIteration()" << std::endl;
 
-  enum class Test : std::uint8_t
-  {
-    A,
-    B,
-    C,
-    D
-  };
-  cm::enum_set<Test, 4> testSet{ Test::A, Test::C, Test::B };
+  EnumSetTest2 testSet{ Test2::A, Test2::C, Test2::B };
 
   if (testSet.size() != 3) {
     ++failed;
@@ -112,17 +166,8 @@ void testEdition()
 {
   std::cout << "testEdition()" << std::endl;
 
-  enum class Test : std::uint8_t
   {
-    A,
-    B,
-    C,
-    D,
-    E
-  };
-
-  {
-    cm::enum_set<Test> testSet{ Test::A, Test::C, Test::B };
+    EnumSetTest testSet{ Test::A, Test::C, Test::B };
 
     auto pos = testSet.insert(Test::E);
     if (!pos.second || testSet.size() != 4 || *(pos.first) != Test::E ||
@@ -144,7 +189,7 @@ void testEdition()
     }
   }
   {
-    cm::enum_set<Test> testSet{ Test::A, Test::C, Test::B };
+    EnumSetTest testSet{ Test::A, Test::C, Test::B };
 
     testSet += { Test::D, Test::E };
 
@@ -173,8 +218,8 @@ void testEdition()
     }
   }
   {
-    cm::enum_set<Test> testSet1{ Test::A, Test::C, Test::B };
-    cm::enum_set<Test> testSet2{ Test::A, Test::D, Test::E };
+    EnumSetTest testSet1{ Test::A, Test::C, Test::B };
+    EnumSetTest testSet2{ Test::A, Test::D, Test::E };
     testSet1.insert(testSet2.cbegin(), testSet2.cend());
 
     std::set<std::uint8_t> reference{ static_cast<std::uint8_t>(Test::A),
@@ -204,8 +249,8 @@ void testEdition()
     }
   }
   {
-    cm::enum_set<Test> testSet1{ Test::A, Test::C, Test::B };
-    cm::enum_set<Test> testSet2{ Test::C, Test::E };
+    EnumSetTest testSet1{ Test::A, Test::C, Test::B };
+    EnumSetTest testSet2{ Test::C, Test::E };
 
     testSet1.flip(Test::A);
     if (testSet1.size() != 2 || testSet1.contains(Test::A)) {
@@ -224,7 +269,7 @@ void testEdition()
     }
   }
   {
-    cm::enum_set<Test> testSet1;
+    EnumSetTest testSet1;
     auto testSet2 = Test::A + Test::C + Test::B;
 
     testSet1.set({ Test::A, Test::C, Test::B });
@@ -264,39 +309,38 @@ void testEdition()
     }
   }
   {
-    using ESet = cm::enum_set<Test, 5>;
-    ESet testSet1;
-    ESet testSet2{ Test::A, Test::C, Test::B };
+    EnumSetTest2 testSet1;
+    EnumSetTest2 testSet2{ Test2::A, Test2::C, Test2::B };
 
     testSet1.set();
     if (testSet1.size() != 5 || testSet1.size() != testSet1.max_size()) {
       ++failed;
     }
-    testSet1.flip({ Test::D, Test::E });
+    testSet1.flip({ Test2::D, Test2::E });
     if (testSet1.size() != 3 || testSet1 != testSet2) {
       ++failed;
     }
-    testSet1.flip(Test::D | Test::E);
-    testSet2 += Test::D + Test::E;
+    testSet1.flip(Test2::D | Test2::E);
+    testSet2 += Test2::D + Test2::E;
     if (testSet1.size() != 5 || testSet1 != testSet2) {
       ++failed;
     }
-    testSet1.flip(Test::E);
-    testSet2 -= Test::E;
+    testSet1.flip(Test2::E);
+    testSet2 -= Test2::E;
     if (testSet1.size() != 4 || testSet1 != testSet2) {
       ++failed;
     }
-    testSet1 ^= { Test::A, Test::B, Test::E, Test::D };
-    testSet2 = { Test::C, Test::E };
+    testSet1 ^= { Test2::A, Test2::B, Test2::E, Test2::D };
+    testSet2 = { Test2::C, Test2::E };
     if (testSet1.size() != 2 || testSet1 != testSet2) {
       ++failed;
     }
-    testSet1 ^= { Test::A, Test::B, Test::E };
-    testSet2 = { Test::A, Test::B, Test::C };
+    testSet1 ^= { Test2::A, Test2::B, Test2::E };
+    testSet2 = { Test2::A, Test2::B, Test2::C };
     if (testSet1.size() != 3 || testSet1 != testSet2) {
       ++failed;
     }
-    testSet2 = Test::A | Test::B | Test::C;
+    testSet2 = Test2::A | Test2::B | Test2::C;
     if (testSet1.size() != 3 || testSet1 != testSet2) {
       ++failed;
     }
@@ -308,15 +352,7 @@ void testChecks()
   std::cout << "testChecks()" << std::endl;
 
   {
-    enum class Test : std::uint8_t
-    {
-      A,
-      B,
-      C,
-      D
-    };
-
-    cm::enum_set<Test> testSet;
+    EnumSetTest testSet;
 
     if (!testSet.empty()) {
       ++failed;
@@ -340,15 +376,7 @@ void testChecks()
     }
   }
   {
-    enum class Test : std::uint8_t
-    {
-      A,
-      B,
-      C,
-      D
-    };
-
-    cm::enum_set<Test, 4> testSet;
+    EnumSetTest2 testSet;
 
     if (!testSet.none()) {
       ++failed;
@@ -357,7 +385,7 @@ void testChecks()
       ++failed;
     }
 
-    testSet = Test::A;
+    testSet = Test2::A;
     if (!testSet.any() || testSet.none() || testSet.all()) {
       ++failed;
     }
@@ -368,16 +396,8 @@ void testChecks()
     }
   }
   {
-    enum class Test : std::uint8_t
-    {
-      A,
-      B,
-      C,
-      D
-    };
-
-    cm::enum_set<Test> testSet1;
-    cm::enum_set<Test> testSet2{ Test::A, Test::C };
+    EnumSetTest testSet1;
+    EnumSetTest testSet2{ Test::A, Test::C };
 
     if (!testSet1.none_of(testSet2) || testSet1.any_of(testSet2) ||
         testSet1.all_of(testSet2)) {
