@@ -1,6 +1,6 @@
 #include <stdio.h>
 
-int main(int argc, const char* argv[])
+int main(int argc, char const* argv[])
 {
   int i = 0;
   for (; i < argc; ++i) {
