@@ -1,6 +1,6 @@
 #include <stdio.h>
 
-int main(int argc, const char* argv[])
+int main(int argc, char const* argv[])
 {
   unsigned int i =
     0; // "i<argc" should produce a "signed/unsigned comparison" warning
