@@ -1,7 +1,7 @@
-void OneFunction();
-void FourFunction();
+void OneFunction(void);
+void FourFunction(void);
 
-void ThreeFunction()
+void ThreeFunction(void)
 {
   static int count = 0;
   if (count == 0) {
