@@ -1,8 +1,9 @@
 #include <bzlib.h>
 #include <stdio.h>
 #include <stdlib.h>
+#include <string.h>
 
-int main()
+int main(void)
 {
   int chunksize = 1024;
   FILE* file = fopen("test.bzip2", "wb");
@@ -19,5 +20,9 @@ int main()
 
   remove("test.bzip2");
 
-  return 0;
+  printf("Found BZip2 version %s, expected version %s\n", BZ2_bzlibVersion(),
+         CMAKE_EXPECTED_BZip2_VERSION);
+
+  return strncmp(BZ2_bzlibVersion(), CMAKE_EXPECTED_BZip2_VERSION,
+                 strlen(CMAKE_EXPECTED_BZip2_VERSION));
 }
