@@ -2,7 +2,7 @@
 #ifdef _WIN32
 __declspec(dllexport)
 #endif
-int empty_c()
+  int empty_c(void)
 {
   return 0;
 }
