@@ -1,30 +1,29 @@
 #pragma once
 
-#include <wrl/client.h>
 #include <ppl.h>
 #include <ppltasks.h>
+#include <wrl/client.h>
 
-namespace DX
+namespace DX {
+inline void ThrowIfFailed(HRESULT hr)
 {
-  inline void ThrowIfFailed(HRESULT hr)
-  {
-    if (FAILED(hr))
-    {
-      // Set a breakpoint on this line to catch Win32 API errors.
-      throw Platform::Exception::CreateException(hr);
-    }
+  if (FAILED(hr)) {
+    // Set a breakpoint on this line to catch Win32 API errors.
+    throw Platform::Exception::CreateException(hr);
   }
+}
 
-  // Function that reads from a binary file asynchronously.
-  inline Concurrency::task<Platform::Array<byte>^> ReadDataAsync(Platform::String^ filename)
-  {
-    using namespace Windows::Storage;
-    using namespace Concurrency;
+// Function that reads from a binary file asynchronously.
+inline Concurrency::task<Platform::Array<byte> ^> ReadDataAsync(
+  Platform::String ^ filename)
+{
+  using namespace Windows::Storage;
+  using namespace Concurrency;
 
-    auto folder = Windows::ApplicationModel::Package::Current->InstalledLocation;
+  auto folder = Windows::ApplicationModel::Package::Current->InstalledLocation;
 
-    return create_task(folder->GetFileAsync(filename)).then([] (StorageFile^ file)
-    {
+  return create_task(folder->GetFileAsync(filename))
+    .then([](StorageFile ^ file) {
 #if !PHONE
       return FileIO::ReadBufferAsync(file);
 #else
@@ -35,11 +34,11 @@ namespace DX
       auto fileBuffer = ref new Streams::Buffer(bufferSize);
       return stream->ReadAsync(fileBuffer, bufferSize, Streams::InputStreamOptions::None);
 #endif
-    }).then([] (Streams::IBuffer^ fileBuffer) -> Platform::Array<byte>^
-    {
+    })
+    .then([](Streams::IBuffer ^ fileBuffer) -> Platform::Array<byte> ^ {
       auto fileData = ref new Platform::Array<byte>(fileBuffer->Length);
       Streams::DataReader::FromBuffer(fileBuffer)->ReadBytes(fileData);
       return fileData;
     });
-  }
+}
 }
