@@ -1,7 +1,6 @@
 /* Distributed under the OSI-approved BSD 3-Clause License.  See accompanying
-   file Copyright.txt or https://cmake.org/licensing for details.  */
-#ifndef cm3p_rhash_h
-#define cm3p_rhash_h
+   file LICENSE.rst or https://cmake.org/licensing for details.  */
+#pragma once
 
 /* Use the LibRHash library configured for CMake.  */
 #include "cmThirdParty.h"
@@ -10,5 +9,3 @@
 #else
 #  include <cmlibrhash/librhash/rhash.h> // IWYU pragma: export
 #endif
-
-#endif
